@@ -1,19 +1,21 @@
 ## code for fast REML computation. key feature is that first and 
 ## second derivatives come at no increase in leading order 
 ## computational cost, relative to evaluation! 
-## (c) Simon N. Wood, 2010-2014
+## (c) Simon N. Wood, 2010-2016
 
 Sl.setup <- function(G) {
 ## Sets up a list representing a block diagonal penalty matrix.
 ## from the object produced by `gam.setup'.
 ## Return object is a list, Sl, with an element for each block.
-## For block, b, Sl[[b]] is a list with the following elemets
+## For block, b, Sl[[b]] is a list with the following elements
+## * repara - should re-parameterization be applied to model matrix etc 
+##            usually false if  non-linear in coefs
 ## * start, stop: start:stop indexes the parameters of this block
 ## * S a list of penalty matrices for the block (dim = stop-start+1)
 ##   - If length(S)==1 then this will be an identity penalty.
 ##   - Otherwise it is a multiple penalty, and an rS list of square
-##     root penalty matrices will be added. S and rS will be projected
-##     into range space of total penalty matrix. 
+##     root penalty matrices will be added. S (if repara) and rS (always) 
+##     will be projected into range space of total penalty matrix. 
 ## * rS sqrt penalty matrices if it's a multiple penalty.
 ## * D a reparameterization matrix for the block
 ##   - Applies to cols/params from start:stop.
@@ -48,10 +50,12 @@ Sl.setup <- function(G) {
         }
         Sl[[b]]$start <- off[ind[1]]
         Sl[[b]]$stop <- Sl[[b]]$start + nr - 1
+	Sl[[b]]$lambda <- rep(1,length(ind)) ## dummy at this stage
       } else { ## singleton
         Sl[[b]] <- list(start=off[ind], stop=off[ind]+nrow(G$S[[ind]])-1,
                         rank=G$rank[ind],S=list(G$S[[ind]]))
         Sl[[b]]$S <- list(G$S[[ind]])
+	Sl[[b]]$lambda <- 1 ## dummy at this stage
       } ## finished singleton
       b <- b + 1 
     } ## finished this block
@@ -66,34 +70,47 @@ Sl.setup <- function(G) {
     if (m>0) {
       Sl[[b]] <- list()
       Sl[[b]]$start <- G$smooth[[i]]$first.para
-      Sl[[b]]$stop <- G$smooth[[i]]$last.para
+      Sl[[b]]$stop <- G$smooth[[i]]$last.para    
+      ## if the smooth has a g.index field it indicates non-linear params,
+      ## in which case re-parameterization will usually break the model!
+      Sl[[b]]$repara <- if (is.null(G$smooth[[i]]$g.index)) TRUE else FALSE 
     }
 
     if (m==0) {} else ## fixed block
     if (m==1) { ## singleton
      
         Sl[[b]]$rank <- G$smooth[[i]]$rank  
-        Sl[[b]]$S <- G$smooth[[i]]$S 
+        Sl[[b]]$S <- G$smooth[[i]]$S
+	Sl[[b]]$lambda <- 1
         b <- b + 1
      
     } else { ## additive block...
       ## first test whether block can *easily* be split up into singletons
       ## easily here means no overlap in penalties 
       Sl[[b]]$S <- G$smooth[[i]]$S
+      Sl[[b]]$lambda <- rep(1,m)
       nb <- nrow(Sl[[b]]$S[[1]])      
-      sbStart <- sbStop <- rep(NA,m)
+      sbdiag <- sbStart <- sbStop <- rep(NA,m)
+      ut <- upper.tri(Sl[[b]]$S[[1]],diag=FALSE) 
       ## overlap testing requires the block ranges 
       for (j in 1:m) { ## get block range for each S[[j]]
+        sbdiag[j] <- sum(abs(Sl[[b]]$S[[j]][ut]))==0 ## is penalty diagonal??
         ir <- range((1:nb)[rowSums(abs(Sl[[b]]$S[[j]]))>0])
-        #ic <- range((1:nb)[colSums(abs(Sl[[b]]$S[[j]]))>0]) ## symmetric not needed
         sbStart[j] <- ir[1];sbStop[j] <- ir[2] ## individual ranges
       } 
       split.ok <- TRUE
       for (j in 1:m) { ## test for overlap
         itot <- rep(FALSE,nb)
-        for (k in 1:m) if (j!=k) itot[sbStart[k]:sbStop[k]] <- TRUE
-        if (sum(itot[sbStart[j]:sbStop[j]])>0) { ## no good, some overlap detected
-          split.ok <- FALSE; break
+        if (all(sbdiag)) { ## it's all diagonal - can allow interleaving
+          for (k in 1:m) if (j!=k) itot[diag(Sl[[b]]$S[[k]])!=0] <- TRUE
+          if (sum(itot[diag(Sl[[b]]$S[[j]])!=0])>0) { ## no good, some overlap detected
+            split.ok <- FALSE; break
+          }
+        } else { ## not diagonal - really need on overlapping blocks
+          for (k in 1:m) if (j!=k) itot[sbStart[k]:sbStop[k]] <- TRUE
+          if (sum(itot[sbStart[j]:sbStop[j]])>0) { ## no good, some overlap detected
+            split.ok <- FALSE; break
+          }
         }
       }
       if (split.ok) { ## can split this block into m separate singleton blocks
@@ -104,13 +121,17 @@ Sl.setup <- function(G) {
           Sl[[b]]$start <- G$smooth[[i]]$first.para + sbStart[j]-1
           Sl[[b]]$stop <- G$smooth[[i]]$first.para + sbStop[j]-1
           Sl[[b]]$rank <- G$smooth[[i]]$rank[j]
+          Sl[[b]]$repara <- TRUE ## signals ok to linearly reparameterize
+          if (!is.null(G$smooth[[i]]$g.index)) { ## then some parameters are non-linear - can't re-param
+            if (any(G$smooth[[i]]$g.index[ind])) Sl[[b]]$repara <- FALSE
+          }
           b <- b + 1
         }
       } else { ## not possible to split
         Sl[[b]]$S <- G$smooth[[i]]$S
-      
+     
+        b <- b + 1 ## next block!!
       } ## additive block finished
-      b <- b + 1 ## next block!!
     } ## additive block finished
   }
 
@@ -147,12 +168,22 @@ Sl.setup <- function(G) {
         Sl[[b]]$D <- t(D*t(U)) ## D <- U%*%diag(D)
         Sl[[b]]$Di <- t(U)/D
         ## so if X is smooth model matrix X%*%D is re-parameterized form 
+        ## crossprod(Sl[[b]]$Di[1:Sl[[b]]$rank]) is the original penalty
         Sl[[b]]$ind <- ind
       }
       ## add penalty square root into E  
-      ind <- (Sl[[b]]$start:Sl[[b]]$stop)[Sl[[b]]$ind]
-      diag(E)[ind] <- 1
-      lambda <- c(lambda,1) ## record corresponding lambda
+      if (Sl[[b]]$repara) { ## then it is just the identity
+        ind <- (Sl[[b]]$start:Sl[[b]]$stop)[Sl[[b]]$ind]
+        diag(E)[ind] <- 1
+        lambda <- c(lambda,1) ## record corresponding lambda
+      } else { ## need scaled root penalty in *original* parameterization
+        D <- Sl[[b]]$Di[1:Sl[[b]]$rank,]
+        D.norm <- norm(D); D <- D/D.norm
+        indc <- Sl[[b]]$start:(Sl[[b]]$start+ncol(D)-1)
+        indr <- Sl[[b]]$start:(Sl[[b]]$start+nrow(D)-1)
+        E[indr,indc] <- D
+        lambda <- c(lambda,1/D.norm^2)
+      }
     } else { ## multiple S block
       ## must be in range space of total penalty...
       Sl[[b]]$rS <- list() ## needed for adaptive re-parameterization
@@ -165,9 +196,12 @@ Sl.setup <- function(G) {
       }
       ind <- 1:Sl[[b]]$rank
       for (j in 1:length(Sl[[b]]$S)) { ## project penalties into range space of total penalty
-        Sl[[b]]$S[[j]] <- t(U[,ind])%*%Sl[[b]]$S[[j]]%*%U[,ind]
-        Sl[[b]]$S[[j]] <- (t(Sl[[b]]$S[[j]]) + Sl[[b]]$S[[j]])/2 ## avoid over-zealous chol sym check
-        Sl[[b]]$rS[[j]] <- mroot(Sl[[b]]$S[[j]],Sl[[b]]$rank)
+        bob <- t(U[,ind])%*%Sl[[b]]$S[[j]]%*%U[,ind]
+        bob <- (t(bob) + bob)/2 ## avoid over-zealous chol sym check
+        if (Sl[[b]]$repara) { ## otherwise want St and E in original parameterization
+          Sl[[b]]$S[[j]] <- bob
+        } 
+        Sl[[b]]$rS[[j]] <- mroot(bob,Sl[[b]]$rank)
       }
       Sl[[b]]$ind <- rep(FALSE,ncol(U))
       Sl[[b]]$ind[ind] <- TRUE ## index penalized within sub-range
@@ -198,11 +232,14 @@ Sl.initial.repara <- function(Sl,X,inverse=FALSE,both.sides=TRUE,cov=TRUE,nt=1)
 ## or, if inverse==TRUE, to apply inverse re-para to parameter vector 
 ## or cov matrix. if inverse is TRUE and both.sides=FALSE then 
 ## re-para only applied to rhs, as appropriate for a choleski factor.
+## If both.sides==FALSE, X is a vector and inverse==FALSE then X is
+## taken as a coefficient vector (so re-para is inverse of that for model
+## matrix...)
   if (length(Sl)==0) return(X) ## nothing to do
   if (inverse) { ## apply inverse re-para
     if (is.matrix(X)) { 
       if (cov) { ## then it's a covariance matrix
-        for (b in 1:length(Sl)) { 
+        for (b in 1:length(Sl)) if (Sl[[b]]$repara) { 
           ind <- Sl[[b]]$start:Sl[[b]]$stop
           if (is.matrix(Sl[[b]]$D)) { 
             if (both.sides) X[ind,] <- if (nt==1) Sl[[b]]$D%*%X[ind,,drop=FALSE] else 
@@ -215,7 +252,7 @@ Sl.initial.repara <- function(Sl,X,inverse=FALSE,both.sides=TRUE,cov=TRUE,nt=1)
           } 
         } 
       } else { ## regular matrix: need to use Di
-         for (b in 1:length(Sl)) { 
+         for (b in 1:length(Sl)) if (Sl[[b]]$repara) { 
           ind <- Sl[[b]]$start:Sl[[b]]$stop
           if (is.matrix(Sl[[b]]$D)) { 
             Di <- if(is.null(Sl[[b]]$Di)) t(Sl[[b]]$D) else Sl[[b]]$Di
@@ -231,13 +268,13 @@ Sl.initial.repara <- function(Sl,X,inverse=FALSE,both.sides=TRUE,cov=TRUE,nt=1)
         } 
       }
     } else { ## it's a parameter vector
-      for (b in 1:length(Sl)) { 
+      for (b in 1:length(Sl)) if (Sl[[b]]$repara) { 
         ind <- Sl[[b]]$start:Sl[[b]]$stop
         if (is.matrix(Sl[[b]]$D)) X[ind] <- Sl[[b]]$D%*%X[ind] else 
         X[ind] <- Sl[[b]]$D*X[ind] 
       }
     }
-  } else for (b in 1:length(Sl)) { ## model matrix re-para
+  } else for (b in 1:length(Sl)) if (Sl[[b]]$repara) { ## model matrix re-para
     ind <- Sl[[b]]$start:Sl[[b]]$stop
     if (is.matrix(X)) { 
       if (is.matrix(Sl[[b]]$D)) { 
@@ -250,8 +287,14 @@ Sl.initial.repara <- function(Sl,X,inverse=FALSE,both.sides=TRUE,cov=TRUE,nt=1)
         X[,ind] <- t(Sl[[b]]$D*t(X[,ind,drop=FALSE])) ## X[,ind]%*%diag(Sl[[b]]$D)
       }
     } else {
-      if (is.matrix(Sl[[b]]$D)) X[ind] <- t(Sl[[b]]$D)%*%X[ind] else 
-      X[ind] <- Sl[[b]]$D*X[ind] 
+      if (both.sides) { ## signalling vector to be treated like model matrix X... 
+        if (is.matrix(Sl[[b]]$D)) X[ind] <- t(Sl[[b]]$D)%*%X[ind] else 
+        X[ind] <- Sl[[b]]$D*X[ind]
+      } else { ## both.sides == FALSE is just a signal that X is a parameter vector
+        if (is.matrix(Sl[[b]]$D)) X[ind] <-
+	  if (is.null(Sl[[b]]$Di)) t(Sl[[b]]$D)%*%X[ind] else Sl[[b]]$Di%*%X[ind] else
+        X[ind] <- X[ind]/Sl[[b]]$D
+      }
     }
   }
   X
@@ -329,8 +372,15 @@ ldetS <- function(Sl,rho,fixed,np,root=FALSE,repara=TRUE,nt=1) {
       } 
       if (root) { 
         ## insert diagonal from block start to end
-        ind <- (Sl[[b]]$start:Sl[[b]]$stop)[Sl[[b]]$ind]
-        diag(E)[ind] <- exp(rho[k.sp]*.5) ## sqrt smoothing param
+        if (Sl[[b]]$repara) {
+          ind <- (Sl[[b]]$start:Sl[[b]]$stop)[Sl[[b]]$ind]
+          diag(E)[ind] <- exp(rho[k.sp]*.5) ## sqrt smoothing param
+        } else { ## root has to be in original parameterization...
+          D <- Sl[[b]]$Di[1:Sl[[b]]$rank,]
+          indc <- Sl[[b]]$start:(Sl[[b]]$start+ncol(D)-1)
+          indr <- Sl[[b]]$start:(Sl[[b]]$start+nrow(D)-1)
+          E[indr,indc] <- D * exp(rho[k.sp]*.5)
+        }
       }
       Sl[[b]]$lambda <- exp(rho[k.sp])
       k.sp <- k.sp + 1 
@@ -355,22 +405,35 @@ ldetS <- function(Sl,rho,fixed,np,root=FALSE,repara=TRUE,nt=1) {
       }
       ## now store the reparameterization information   
       if (repara) {   
-        rp[[k.rp]] <- list(block =b,ind = (Sl[[b]]$start:Sl[[b]]$stop)[Sl[[b]]$ind],Qs = grp$Qs)
+        rp[[k.rp]] <- list(block =b,ind = (Sl[[b]]$start:Sl[[b]]$stop)[Sl[[b]]$ind],Qs = grp$Qs,repara=Sl[[b]]$repara)
         k.rp <- k.rp + 1
         for (i in 1:m) {
           Sl[[b]]$Srp[[i]] <- Sl[[b]]$lambda[i]*grp$rS[[i]]%*%t(grp$rS[[i]])
         }
       }
       k.sp <- k.sp + m
-      if (root) { ## unpack the square root E'E
-        ic <- Sl[[b]]$start:(Sl[[b]]$start+ncol(grp$E)-1)
-        ir <- Sl[[b]]$start:(Sl[[b]]$start+nrow(grp$E)-1)
-        E[ir,ic] <- grp$E      
-        Sl[[b]]$St <- crossprod(grp$E)
-      } else {  
-        ## gam.reparam always returns root penalty in E, but 
-        ## ldetSblock returns penalty in E if root==FALSE
-        Sl[[b]]$St <- if (repara) crossprod(grp$E) else grp$E
+      if (Sl[[b]]$repara) {
+        if (root) { ## unpack the square root E'E
+          ic <- Sl[[b]]$start:(Sl[[b]]$start+ncol(grp$E)-1)
+          ir <- Sl[[b]]$start:(Sl[[b]]$start+nrow(grp$E)-1)
+          E[ir,ic] <- grp$E      
+          Sl[[b]]$St <- crossprod(grp$E)
+        } else {  
+          ## gam.reparam always returns root penalty in E, but 
+          ## ldetSblock returns penalty in E if root==FALSE
+          Sl[[b]]$St <- if (repara) crossprod(grp$E) else grp$E
+        }
+      } else { ## square root block and St need to be in original parameterization...
+        Sl[[b]]$St <- Sl[[b]]$lambda[1]*Sl[[b]]$S[[1]]
+        for (i in 2:m) {
+          Sl[[b]]$St <- Sl[[b]]$St + Sl[[b]]$lambda[i]*Sl[[b]]$S[[i]]
+        }
+        if (root) {
+          Eb <- t(mroot(Sl[[b]]$St,Sl[[b]]$rank))
+          indc <- Sl[[b]]$start:(Sl[[b]]$start+ncol(Eb)-1)
+          indr <- Sl[[b]]$start:(Sl[[b]]$start+nrow(Eb)-1)
+          E[indr,indc] <- Eb
+        } 
       }   
     } ## end of multi-S block branch
   } ## end of block loop
@@ -409,20 +472,20 @@ Sl.repara <- function(rp,X,inverse=FALSE,both.sides=TRUE) {
   nr <- length(rp);if (nr==0) return(X)
   if (inverse) {
     if (is.matrix(X)) { ## X is a cov matrix
-      for (i in 1:nr) {
+      for (i in 1:nr) if (rp[[i]]$repara) {
         if (both.sides) X[rp[[i]]$ind,]  <- 
                      rp[[i]]$Qs %*% X[rp[[i]]$ind,,drop=FALSE]
         X[,rp[[i]]$ind]  <- 
                      X[,rp[[i]]$ind,drop=FALSE] %*% t(rp[[i]]$Qs)
       }
     } else { ## X is a vector
-     for (i in 1:nr) X[rp[[i]]$ind]  <- rp[[i]]$Qs %*% X[rp[[i]]$ind]
+     for (i in 1:nr) if (rp[[i]]$repara) X[rp[[i]]$ind]  <- rp[[i]]$Qs %*% X[rp[[i]]$ind]
     }
   } else { ## apply re-para to X
     if (is.matrix(X)) {
-      for (i in 1:nr) X[,rp[[i]]$ind]  <- X[,rp[[i]]$ind]%*%rp[[i]]$Qs
+      for (i in 1:nr) if (rp[[i]]$repara) X[,rp[[i]]$ind]  <- X[,rp[[i]]$ind]%*%rp[[i]]$Qs
     } else {
-      for (i in 1:nr) X[rp[[i]]$ind]  <- t(rp[[i]]$Qs) %*% X[rp[[i]]$ind]
+      for (i in 1:nr) if (rp[[i]]$repara) X[rp[[i]]$ind]  <- t(rp[[i]]$Qs) %*% X[rp[[i]]$ind]
     }
   }
   X
@@ -440,9 +503,15 @@ Sl.mult <- function(Sl,A,k = 0,full=TRUE) {
     B <- A*0
     for (b in 1:nb) { ## block loop
       if (length(Sl[[b]]$S)==1) { ## singleton
-        ind <- (Sl[[b]]$start:Sl[[b]]$stop)[Sl[[b]]$ind]
-        if (Amat) B[ind,] <- Sl[[b]]$lambda*A[ind,] else
-                  B[ind] <- Sl[[b]]$lambda*A[ind]
+        ind <- Sl[[b]]$start:Sl[[b]]$stop
+        if (Sl[[b]]$repara) {
+          ind <- ind[Sl[[b]]$ind]
+          if (Amat) B[ind,] <- Sl[[b]]$lambda*A[ind,] else
+                    B[ind] <- Sl[[b]]$lambda*A[ind]
+        } else { ## original penalty has to be applied 
+          if (Amat) B[ind,] <- Sl[[b]]$lambda*Sl[[b]]$S[[1]] %*% A[ind,] else
+                    B[ind] <- Sl[[b]]$lambda*Sl[[b]]$S[[1]] %*% A[ind]
+        }
       } else { ## multi-S block
         ind <- (Sl[[b]]$start:Sl[[b]]$stop)[Sl[[b]]$ind]
         if (Amat) B[ind,] <- Sl[[b]]$St %*% A[ind,] else
@@ -457,30 +526,43 @@ Sl.mult <- function(Sl,A,k = 0,full=TRUE) {
         j <- j + 1
         if (j==k) { ## found block
           if (length(Sl[[b]]$S)==1) { ## singleton
-            ind <- (Sl[[b]]$start:Sl[[b]]$stop)[Sl[[b]]$ind]
-            if (full) { ## return zero answer with all zeroes in place
-              B <- A*0
-              if (Amat) B[ind,] <- Sl[[b]]$lambda*A[ind,] else  
-                        B[ind] <- Sl[[b]]$lambda*A[ind]
-              A <- B
-            } else { ## strip zero rows from answer
-              if (Amat) A <- Sl[[b]]$lambda*A[ind,] else
-                        A <- as.numeric(Sl[[b]]$lambda*A[ind])
-            }
+            ind <- Sl[[b]]$start:Sl[[b]]$stop
+            if (Sl[[b]]$repara) {
+              ind <- ind[Sl[[b]]$ind]
+              if (full) { ## return zero answer with all zeroes in place
+                B <- A*0
+                if (Amat) B[ind,] <- Sl[[b]]$lambda*A[ind,] else  
+                          B[ind] <- Sl[[b]]$lambda*A[ind]
+                A <- B
+              } else { ## strip zero rows from answer
+                if (Amat) A <- Sl[[b]]$lambda*A[ind,] else
+                          A <- as.numeric(Sl[[b]]$lambda*A[ind])
+              } 
+            } else { ## not reparamterized version 
+              if (full) {
+                B <- A*0
+                if (Amat) B[ind,] <- Sl[[b]]$lambda*Sl[[b]]$S[[1]]%*%A[ind,] else  
+                          B[ind] <- Sl[[b]]$lambda*Sl[[b]]$S[[1]]%*%A[ind]
+                A <- B
+              } else {
+                if (Amat) A <- Sl[[b]]$lambda*Sl[[b]]$S[[1]]%*%A[ind,] else
+                          A <- as.numeric(Sl[[b]]$lambda*Sl[[b]]$S[[1]]%*%A[ind])
+              }
+            } ## not repara
           } else { ## multi-S block
-            ind <- (Sl[[b]]$start:Sl[[b]]$stop)[Sl[[b]]$ind]
+            ind <- if (Sl[[b]]$repara) (Sl[[b]]$start:Sl[[b]]$stop)[Sl[[b]]$ind] else Sl[[b]]$start:Sl[[b]]$stop
             if (full) { ## return zero answer with all zeroes in place
               B <- A*0
               if (Amat) {
-                B[ind,] <- if (is.null(Sl[[b]]$Srp)) Sl[[b]]$lambda[i]*(Sl[[b]]$S[[i]]%*%A[ind,]) else 
+                B[ind,] <- if (is.null(Sl[[b]]$Srp)||!Sl[[b]]$repara) Sl[[b]]$lambda[i]*(Sl[[b]]$S[[i]]%*%A[ind,]) else 
                                                      Sl[[b]]$Srp[[i]]%*%A[ind,]
               } else {
-                B[ind] <- if (is.null(Sl[[b]]$Srp)) Sl[[b]]$lambda[i]*(Sl[[b]]$S[[i]]%*%A[ind]) else 
+                B[ind] <- if (is.null(Sl[[b]]$Srp)||!Sl[[b]]$repara) Sl[[b]]$lambda[i]*(Sl[[b]]$S[[i]]%*%A[ind]) else 
                                                     Sl[[b]]$Srp[[i]]%*%A[ind]
               }
               A <- B
             } else { ## strip zero rows from answer
-              if (is.null(Sl[[b]]$Srp)) {
+              if (is.null(Sl[[b]]$Srp)||!Sl[[b]]$repara) {
                 A <- if (Amat)  Sl[[b]]$lambda[i]*(Sl[[b]]$S[[i]]%*%A[ind,]) else  
                                 Sl[[b]]$lambda[i]*as.numeric(Sl[[b]]$S[[i]]%*%A[ind])
               } else {
@@ -510,24 +592,38 @@ Sl.termMult <- function(Sl,A,full=FALSE,nt=1) {
   for (b in 1:nb) { ## block loop
     if (length(Sl[[b]]$S)==1) { ## singleton
       k <- k + 1
-      ind <- (Sl[[b]]$start:Sl[[b]]$stop)[Sl[[b]]$ind]
-      if (full) { ## return zero answer with all zeroes in place
-        B <- A*0
-        if (Amat) B[ind,] <- Sl[[b]]$lambda*A[ind,,drop=FALSE] else  
-                  B[ind] <- Sl[[b]]$lambda*A[ind]
-        SA[[k]] <- B
-      } else { ## strip zero rows from answer
-        if (Amat) SA[[k]] <- Sl[[b]]$lambda*A[ind,,drop=FALSE] else
-                  SA[[k]] <- as.numeric(Sl[[b]]$lambda*A[ind])
-        attr(SA[[k]],"ind") <- ind
+      ind <- Sl[[b]]$start:Sl[[b]]$stop
+      if (Sl[[b]]$repara) {
+        ind <- ind[Sl[[b]]$ind]
+        if (full) { ## return zero answer with all zeroes in place
+          B <- A*0
+          if (Amat) B[ind,] <- Sl[[b]]$lambda*A[ind,,drop=FALSE] else  
+                    B[ind] <- Sl[[b]]$lambda*A[ind]
+          SA[[k]] <- B
+        } else { ## strip zero rows from answer
+          if (Amat) SA[[k]] <- Sl[[b]]$lambda*A[ind,,drop=FALSE] else
+                    SA[[k]] <- as.numeric(Sl[[b]]$lambda*A[ind])
+          attr(SA[[k]],"ind") <- ind
+        }
+      } else {
+        if (full) { ## return zero answer with all zeroes in place
+          B <- A*0
+          if (Amat) B[ind,] <- Sl[[b]]$lambda*Sl[[b]]$S[[1]]%*%A[ind,,drop=FALSE] else  
+                    B[ind] <- Sl[[b]]$lambda*Sl[[b]]$S[[1]]%*%A[ind]
+          SA[[k]] <- B
+        } else { ## strip zero rows from answer
+          if (Amat) SA[[k]] <- Sl[[b]]$lambda*Sl[[b]]$S[[1]]%*%A[ind,,drop=FALSE] else
+                    SA[[k]] <- as.numeric(Sl[[b]]$lambda*Sl[[b]]$S[[1]]%*%A[ind])
+          attr(SA[[k]],"ind") <- ind
+        }
       }
     } else { ## multi-S block
-      ind <- (Sl[[b]]$start:Sl[[b]]$stop)[Sl[[b]]$ind]
+      ind <- if (Sl[[b]]$repara) (Sl[[b]]$start:Sl[[b]]$stop)[Sl[[b]]$ind] else Sl[[b]]$start:Sl[[b]]$stop
       for (i in 1:length(Sl[[b]]$S)) { ## work through S terms
         k <- k + 1
         if (full) { ## return answer with all zeroes in place
           B <- A*0
-          if (is.null(Sl[[b]]$Srp)) {
+          if (is.null(Sl[[b]]$Srp)||!Sl[[b]]$repara) {
             if (Amat) { 
               B[ind,] <- if (nt==1)  Sl[[b]]$lambda[i]*(Sl[[b]]$S[[i]]%*%A[ind,,drop=FALSE]) else 
                          Sl[[b]]$lambda[i]*pmmult(Sl[[b]]$S[[i]],A[ind,,drop=FALSE],nt=nt) 
@@ -540,7 +636,7 @@ Sl.termMult <- function(Sl,A,full=FALSE,nt=1) {
           }
           SA[[k]] <- B
         } else { ## strip zero rows from answer
-          if (is.null(Sl[[b]]$Srp)) {
+          if (is.null(Sl[[b]]$Srp)||!Sl[[b]]$repara) {
             if (Amat) {
               SA[[k]] <- if (nt==1)  Sl[[b]]$lambda[i]*(Sl[[b]]$S[[i]]%*%A[ind,,drop=FALSE]) else
                          Sl[[b]]$lambda[i]*pmmult(Sl[[b]]$S[[i]],A[ind,,drop=FALSE],nt=nt)
@@ -644,7 +740,9 @@ Sl.iftChol <- function(Sl,XX,R,d,beta,piv) {
                                  sum(db[,j]*Skb[[k]]))                                  
     }
   }
-  list(bSb=sum(beta*Sb),bSb1=bSb1,bSb2=bSb2,d1b=db,rss1=rss1,rss2=rss2)
+  list(bSb=sum(beta*Sb),bSb1=bSb1,bSb2=bSb2,
+       d1b=db ## BUG - this needs transforming as coef - here, or where used
+       ,rss1=rss1,rss2=rss2)
 } ## end Sl.iftChol
 
 
@@ -655,7 +753,7 @@ Sl.fitChol <- function(Sl,XX,f,rho,yy=0,L=NULL,rho0=0,log.phi=0,phi.fixed=TRUE,n
 ## yy = y'Wy in order to get derivsatives w.r.t. phi. 
   
   rho <- if (is.null(L)) rho + rho0 else L%*%rho + rho0
-
+  if (length(rho)<length(rho0)) rho <- rho0 ## ncol(L)==0 or length(rho)==0
   ## get log|S|_+ without stability transform... 
   fixed <- rep(FALSE,length(rho))
   ldS <- ldetS(Sl,rho,fixed,np=ncol(XX),root=FALSE,repara=FALSE,nt=nt)
@@ -668,7 +766,8 @@ Sl.fitChol <- function(Sl,XX,f,rho,yy=0,L=NULL,rho0=0,log.phi=0,phi.fixed=TRUE,n
   d[ind] <- 1;d[!ind] <- sqrt(d[!ind])
   #XXp <- t(XXp/d)/d ## diagonally precondition
   R <- if (nt>1) pchol(t(XXp/d)/d,nt) else suppressWarnings(chol(t(XXp/d)/d,pivot=TRUE))
-  r <- Rrank(R);p <- ncol(XXp)
+  r <- min(attr(R,"rank"),Rrank(R))
+  p <- ncol(XXp)
   piv <- attr(R,"pivot") #;rp[rp] <- 1:p
   if (r<p) { ## drop rank deficient terms...
     R <- R[1:r,1:r]
@@ -701,7 +800,8 @@ Sl.fitChol <- function(Sl,XX,f,rho,yy=0,L=NULL,rho0=0,log.phi=0,phi.fixed=TRUE,n
     rss.bSb <- yy - sum(beta*f) ## use identity ||y-Xb|| + b'Sb = y'y - b'X'y (b is minimizer)
     reml1[n+1] <- (-rss.bSb/phi + nobs - Mp)/2
     d <- c(-(dift$rss1[!fixed] + dift$bSb1[!fixed]),rss.bSb)/(2*phi)
-    reml2 <- rbind(cbind(reml2,d[1:n]),d)
+    reml2 <- rbind(cbind(reml2,d[1:n]),d) 
+    if (!is.null(L)) L <- rbind(cbind(L,rep(0,nrow(L))),c(rep(0,ncol(L)),1))
   }
 
   if (!is.null(L)) {
@@ -711,22 +811,23 @@ Sl.fitChol <- function(Sl,XX,f,rho,yy=0,L=NULL,rho0=0,log.phi=0,phi.fixed=TRUE,n
   uconv.ind <- (abs(reml1) > tol)|(abs(diag(reml2))>tol)
   hess <- reml2
   grad <- reml1
-  if (sum(uconv.ind)!=ncol(reml2)) { 
-    reml1 <- reml1[uconv.ind]
-    reml2 <- reml2[uconv.ind,uconv.ind]
-  }
+  if (length(grad)>0) {
+    if (sum(uconv.ind)!=ncol(reml2)) { 
+      reml1 <- reml1[uconv.ind]
+      reml2 <- reml2[uconv.ind,uconv.ind]
+    }
 
-  er <- eigen(reml2,symmetric=TRUE)
-  er$values <- abs(er$values)
-  me <- max(er$values)*.Machine$double.eps^.5
-  er$values[er$values<me] <- me
-  step <- rep(0,length(uconv.ind))
-  step[uconv.ind] <- -er$vectors%*%((t(er$vectors)%*%reml1)/er$values)
-
-  ## limit the step length...
-  ms <- max(abs(step))
-  if (ms>4) step <- 4*step/ms
+    er <- eigen(reml2,symmetric=TRUE)
+    er$values <- abs(er$values)
+    me <- max(er$values)*.Machine$double.eps^.5
+    er$values[er$values<me] <- me
+    step <- rep(0,length(uconv.ind))
+    step[uconv.ind] <- -er$vectors%*%((t(er$vectors)%*%reml1)/er$values)
 
+    ## limit the step length...
+    ms <- max(abs(step))
+    if (ms>4) step <- 4*step/ms
+  } else step <- 0
   ## return the coefficient estimate, the reml grad and the Newton step...
   list(beta=beta,grad=grad,step=step,db=dift$d1b,PP=PP,R=R,piv=piv,rank=r,
        hess=hess,ldetS=ldS$ldetS,ldetXXS=ldetXXS)
@@ -840,7 +941,13 @@ fast.REML.fit <- function(Sl,X,y,rho,L=NULL,rho.0=NULL,log.phi=0,phi.fixed=TRUE,
   ## idea in following is only to exclude terms with zero first and second derivative
   ## from optimization, as it is only these that slow things down if included...  
   uconv.ind <- (abs(grad) > reml.scale*conv.tol*.1)|(abs(grad2)>reml.scale*conv.tol*.1)
- 
+  ## if all appear converged at this stage, then there is probably something wrong,
+  ## but reset anyway to see if situation can be recovered. If we don't do this then
+  ## need to abort immediately, otherwise fails trying to eigen a 0 by 0 matrix
+  if (sum(uconv.ind)==0) {
+    warning("Possible divergence detected in fast.REML.fit",call.=FALSE,immediate.=TRUE)
+    uconv.ind <- rep(TRUE,length(grad)) 
+  }
   step.failed <- FALSE
   for (iter in 1:200) { ## the Newton loop
     ## Work only with unconverged (much quicker under indefiniteness)
@@ -1038,7 +1145,6 @@ Sl.postproc <- function(Sl,fit,undrop,X0,cov=FALSE,scale = -1,L,nt=nt) {
     }
     Vp <- PP * scale ## cov matrix
     ## sp uncertainty correction... 
-    ## BUG: possibility of L ignored here.
     if (!is.null(L)) d1b <- d1b%*%L
     M <- ncol(d1b) 
     ev <- eigen(fit$outer.info$hess,symmetric=TRUE)
