@@ -32,7 +32,7 @@ fix.family.qf <- function(fam) {
     }
   } else if (family=="binomial") {
     fam$qf <- function(p,mu,wt,scale) {
-      qbinom(p,wt,mu)/wt
+      qbinom(p,wt,mu)/(wt + as.numeric(wt==0))
     }
   } else if (family=="Gamma") {
     fam$qf <- function(p,mu,wt,scale) {
@@ -59,7 +59,7 @@ fix.family.rd <- function(fam) {
     }
   } else if (family=="binomial") {
     fam$rd <- function(mu,wt,scale) {
-      rbinom(mu,wt,mu)/wt
+      rbinom(mu,wt,mu)/(wt + as.numeric(wt==0))
     }
   } else if (family=="Gamma") {
     fam$rd <- function(mu,wt,scale) {
@@ -111,7 +111,7 @@ qq.gam <- function(object, rep=0, level=.9,s.rep=10,
   if (rep > 0) { ## simulate quantiles
     fam <- fix.family.rd(object$family)
     if (!is.null(fam$rd)) {
-      d <- rep(0,0)
+      ##d <- rep(0,0)
       ## simulate deviates... 
       dm <- matrix(0,n,rep)
       for (i in 1:rep) { 
@@ -169,7 +169,7 @@ qq.gam <- function(object, rep=0, level=.9,s.rep=10,
     points(Dq,sort(D),pch=pch,...)
     return(invisible(Dq))
   } else qqnorm(D,ylab=ylab,pch=pch,...)
-}
+} ## qq.gam
 
 
 k.check <- function(b,subsample=5000,n.rep=400) {
@@ -191,15 +191,18 @@ k.check <- function(b,subsample=5000,n.rep=400) {
   } else modf <- b$model
   nr <- length(rsd)
   for (k in 1:m) { ## work through smooths
-    dat <- as.data.frame(ExtractData(b$smooth[[k]],modf,NULL)$data)
+    ok <- TRUE
+    b$smooth[[k]]$by <- "NA" ## can't deal with by variables
+    dat <- ExtractData(b$smooth[[k]],modf,NULL)$data
+    if (!is.null(attr(dat,"index"))||!is.null(attr(dat[[1]],"matrix"))||is.matrix(dat[[1]])) ok <- FALSE
+    if (ok) dat <- as.data.frame(dat)
     snames[k] <- b$smooth[[k]]$label
     ind <- b$smooth[[k]]$first.para:b$smooth[[k]]$last.para
     kc[k] <- length(ind)
     edf[k] <- sum(b$edf[ind]) 
     nc <- b$smooth[[k]]$dim
-    ok <- TRUE
+    if (ok && ncol(dat)>nc) dat <- dat[,1:nc,drop=FALSE] ## drop any by variables
     for (j in 1:nc) if (is.factor(dat[[j]])) ok <- FALSE 
-    if (!is.null(attr(dat[[1]],"matrix"))) ok <- FALSE
     if (!ok) {
       p.val[k] <- v.obs[k] <- NA ## can't do this test with summation convention/factors
     } else { ## normal term
@@ -256,13 +259,15 @@ gam.check <- function(b, old.style=FALSE,
                       k.sample=5000,k.rep=200,
 		      ## arguments passed to qq.gam() {w/o warnings !}:
 		      rep=0, level=.9, rl.col=2, rep.col="gray80", ...)
-# takes a fitted gam object and produces some standard diagnostic plots
+## takes a fitted gam object and produces some standard diagnostic plots
 {
   type <- match.arg(type)
   resid <- residuals(b, type=type)
-  linpred <- napredict(b$na.action, b$linear.predictors)
+  linpred <- if (is.matrix(b$linear.predictors)&&!is.matrix(resid)) 
+             napredict(b$na.action, b$linear.predictors[,1]) else 
+             napredict(b$na.action, b$linear.predictors)
 ##  if (b$method%in%c("GCV","GACV","UBRE","REML","ML","P-ML","P-REML","mle.REML","mle.ML","PQL")) { 
-    old.par<-par(mfrow=c(2,2))
+    if (is.null(.Platform$GUI) || .Platform$GUI != "RStudio") old.par <- par(mfrow=c(2,2))
     if (old.style)
       qqnorm(resid,...)
     else
@@ -270,7 +275,9 @@ gam.check <- function(b, old.style=FALSE,
     plot(linpred, resid,main="Resids vs. linear pred.",
          xlab="linear predictor",ylab="residuals",...)
     hist(resid,xlab="Residuals",main="Histogram of residuals",...)
-    plot(fitted(b), napredict(b$na.action, b$y),
+    fv <- if (inherits(b$family,"extended.family")) predict(b,type="response") else fitted(b)
+    if (is.matrix(fv)&&!is.matrix(b$y)) fv <- fv[,1]
+    plot(fv, napredict(b$na.action, b$y),
          xlab="Fitted Values",ylab="Response",main="Response vs. Fitted Values",...)
     if (!(b$method%in%c("GCV","GACV","UBRE","REML","ML","P-ML","P-REML","fREML"))) { ## gamm `gam' object
        par(old.par)
@@ -300,13 +307,17 @@ gam.check <- function(b, old.style=FALSE,
          
         if (!b$mgcv.conv$fully.converged)
         cat(" by steepest\ndescent step failure.\n") else cat(".\n")
-        cat("The RMS",b$method,"score gradiant at convergence was",b$mgcv.conv$rms.grad,".\n")
+        cat("The RMS",b$method,"score gradient at convergence was",b$mgcv.conv$rms.grad,".\n")
         if (b$mgcv.conv$hess.pos.def)
         cat("The Hessian was positive definite.\n") else cat("The Hessian was not positive definite.\n")
-        cat("The estimated model rank was ",b$mgcv.conv$rank,
-                   " (maximum possible: ",b$mgcv.conv$full.rank,")\n",sep="")
+        #cat("The estimated model rank was ",b$mgcv.conv$rank,
+        #           " (maximum possible: ",b$mgcv.conv$full.rank,")\n",sep="")
       }
     }
+    if (!is.null(b$rank)) {
+      cat("Model rank = ",b$rank,"/",length(b$coefficients),"\n")
+    }
+
     cat("\n")
     ## now check k
     kchck <- k.check(b,subsample=k.sample,n.rep=k.rep)
@@ -315,7 +326,7 @@ gam.check <- function(b, old.style=FALSE,
       cat("indicate that k is too low, especially if edf is close to k\'.\n\n")
       printCoefmat(kchck,digits=3);
     }
-    par(old.par)
+    if (is.null(.Platform$GUI) ||.Platform$GUI != "RStudio") par(old.par)
 ##  } else plot(linpred,resid,xlab="linear predictor",ylab="residuals",...)
 } ## end of gam.check
 
@@ -337,6 +348,7 @@ plot.random.effect <- function(x,P=NULL,data=NULL,label="",se1.mult=1,se2.mult=2
       X <- diag(p)   # prediction matrix for this term
       if (is.null(xlab)) xlabel<- "Gaussian quantiles" else xlabel <- xlab
       if (is.null(ylab)) ylabel <- "effects" else ylabel <- ylab
+      if (!is.null(main)) label <- main
       return(list(X=X,scale=FALSE,se=FALSE,raw=raw,xlab=xlabel,ylab=ylabel,
              main=label))
 
@@ -426,13 +438,15 @@ plot.sos.smooth <- function(x,P=NULL,data=NULL,label="",se1.mult=1,se2.mult=2,
                      partial.resids=FALSE,rug=TRUE,se=TRUE,scale=-1,n=100,n2=40,
                      pers=FALSE,theta=30,phi=30,jit=FALSE,xlab=NULL,ylab=NULL,main=NULL,
                      ylim=NULL,xlim=NULL,too.far=0.1,shade=FALSE,shade.col="gray80",
-                     shift=0,trans=I,by.resids=FALSE,scheme=0,...) {
+                     shift=0,trans=I,by.resids=FALSE,scheme=0,hcolors=heat.colors(100),
+                     contour.col=3,...) {
 ## plot method function for sos.smooth terms
   if (scheme>1) return(plot.mgcv.smooth(x,P=P,data=data,label=label,se1.mult=se1.mult,se2.mult=se2.mult,
                      partial.resids=partial.resids,rug=rug,se=se,scale=scale,n=n,n2=n2,
                      pers=pers,theta=theta,phi=phi,jit=jit,xlab=xlab,ylab=ylab,main=main,
                      ylim=ylim,xlim=xlim,too.far=too.far,shade=shade,shade.col=shade.col,
-                     shift=shift,trans=trans,by.resids=by.resids,scheme=scheme-2,...))
+                     shift=shift,trans=trans,by.resids=by.resids,scheme=scheme-2,
+                     hcolors=hcolors,contour.col=contour.col,...))
   ## convert location of pole in plotting grid to radians
   phi <- phi*pi/180
   theta <- theta*pi/180
@@ -472,6 +486,7 @@ plot.sos.smooth <- function(x,P=NULL,data=NULL,label="",se1.mult=1,se2.mult=2,
  
     dat <- data.frame(la=um$la*180/pi,lo=um$lo*180/pi)
     names(dat) <- x$term
+    if (x$by!="NA") dat[[x$by]] <- la*0+1    
 
     X <- PredictMat(x,dat)   # prediction matrix for this term
 
@@ -491,7 +506,7 @@ plot.sos.smooth <- function(x,P=NULL,data=NULL,label="",se1.mult=1,se2.mult=2,
     if (scheme == 0) {
       col <- 1# "lightgrey 
       zz[P$ind] <- P$fit
-      image(P$xm,P$ym,matrix(zz,m,m),col=heat.colors(100),axes=FALSE,xlab="",ylab="",...)
+      image(P$xm,P$ym,matrix(zz,m,m),col=hcolors,axes=FALSE,xlab="",ylab="",...)
       if (rug) { 
         if (is.null(list(...)[["pch"]])) points(P$raw$x,P$raw$y,pch=".",...) else
         points(P$raw$x,P$raw$y,...)
@@ -501,7 +516,7 @@ plot.sos.smooth <- function(x,P=NULL,data=NULL,label="",se1.mult=1,se2.mult=2,
       zz[P$ind] <- P$lo
       contour(P$xm,P$ym,matrix(zz,m,m),add=TRUE,levels=c(-8:9*20),col=col,...)
       zz[P$ind] <- P$fit
-      contour(P$xm,P$ym,matrix(zz,m,m),add=TRUE,col=3,...)
+      contour(P$xm,P$ym,matrix(zz,m,m),add=TRUE,col=contour.col,...)
     } else if (scheme == 1) {
       col <- 1 
       zz[P$ind] <- P$fit
@@ -660,7 +675,6 @@ plot.fs.interaction <- function(x,P=NULL,data=NULL,label="",se1.mult=1,se2.mult=
     fac <- rep(x$flev,rep(n,nf))
     dat <- data.frame(fac,xx)
     names(dat) <- c(x$fterm,x$base$term)
-#    X <- Predict.matrix.fs.interaction(x,dat)
     X <- PredictMat(x,dat)
     if (is.null(xlab)) xlabel <- x$base$term else xlabel <- xlab
     if (is.null(ylab)) ylabel <- label else ylabel <- ylab
@@ -668,7 +682,8 @@ plot.fs.interaction <- function(x,P=NULL,data=NULL,label="",se1.mult=1,se2.mult=
              main="",x=xx,n=n,nf=nf))
   } else { ## produce the plot
     ind <- 1:P$n
-    plot(P$x[ind],P$fit[ind],ylim=range(P$fit),xlab=P$xlab,ylab=P$ylab,type="l")
+    if(is.null(ylim)) ylim <- range(P$fit) 
+    plot(P$x[ind],P$fit[ind],ylim=ylim,xlab=P$xlab,ylab=P$ylab,type="l",...)
     if (P$nf>1) for (i in 2:P$nf) {
       ind <- ind + P$n
       if (scheme==0) lines(P$x,P$fit[ind],lty=i,col=i) else 
@@ -681,7 +696,8 @@ plot.mgcv.smooth <- function(x,P=NULL,data=NULL,label="",se1.mult=1,se2.mult=2,
                      partial.resids=FALSE,rug=TRUE,se=TRUE,scale=-1,n=100,n2=40,
                      pers=FALSE,theta=30,phi=30,jit=FALSE,xlab=NULL,ylab=NULL,main=NULL,
                      ylim=NULL,xlim=NULL,too.far=0.1,shade=FALSE,shade.col="gray80",
-                     shift=0,trans=I,by.resids=FALSE,scheme=0,...) {
+                     shift=0,trans=I,by.resids=FALSE,scheme=0,hcolors=heat.colors(50),
+                     contour.col=3,...) {
 ## default plot method for smooth objects `x' inheriting from "mgcv.smooth"
 ## `x' is a smooth object, usually part of a `gam' fit. It has an attribute
 ##     'coefficients' containg the coefs for the smooth, but usually these
@@ -851,17 +867,17 @@ plot.mgcv.smooth <- function(x,P=NULL,data=NULL,label="",se1.mult=1,se2.mult=2,
             if (min.r < ylimit[1]) ylimit[1] <- min.r
           }
         }
-        if (!is.null(ylim)) ylimit <- ylim
+        ylimit <- if (is.null(ylim)) ylimit <- ylimit + shift else ylim
          
         ## plot the smooth... 
         if (shade) { 
-          plot(P$x,trans(P$fit+shift),type="n",xlab=P$xlab,ylim=trans(ylimit+shift),
+          plot(P$x,trans(P$fit+shift),type="n",xlab=P$xlab,ylim=trans(ylimit),
                  xlim=P$xlim,ylab=P$ylab,main=P$main,...)
           polygon(c(P$x,P$x[n:1],P$x[1]),
                     trans(c(ul,ll[n:1],ul[1])+shift),col = shade.col,border = NA)
           lines(P$x,trans(P$fit+shift),...)
         } else { ## ordinary plot 
-          plot(P$x,trans(P$fit+shift),type="l",xlab=P$xlab,ylim=trans(ylimit+shift),xlim=P$xlim,
+          plot(P$x,trans(P$fit+shift),type="l",xlab=P$xlab,ylim=trans(ylimit),xlim=P$xlim,
                  ylab=P$ylab,main=P$main,...)
           if (is.null(list(...)[["lty"]])) { 
             lines(P$x,trans(ul+shift),lty=2,...)
@@ -893,10 +909,11 @@ plot.mgcv.smooth <- function(x,P=NULL,data=NULL,label="",se1.mult=1,se2.mult=2,
         if (scheme == 1) { ## perspective plot 
           persp(P$x,P$y,matrix(trans(P$fit+shift),n2,n2),xlab=P$xlab,ylab=P$ylab,
                   zlab=P$main,ylim=P$ylim,xlim=P$xlim,theta=theta,phi=phi,...)
-        } else if (scheme==2) {
+        } else if (scheme==2||scheme==3) {
+          if (scheme==3) hcolors <- grey(0:50/50)
           image(P$x,P$y,matrix(trans(P$fit+shift),n2,n2),xlab=P$xlab,ylab=P$ylab,
-                  main=P$main,xlim=P$xlim,ylim=P$ylim,col=heat.colors(50),...)
-          contour(P$x,P$y,matrix(trans(P$fit+shift),n2,n2),add=TRUE,col=3,...)
+                  main=P$main,xlim=P$xlim,ylim=P$ylim,col=hcolors,...)
+          contour(P$x,P$y,matrix(trans(P$fit+shift),n2,n2),add=TRUE,col=contour.col,...)
           if (rug) {  
             if (is.null(list(...)[["pch"]])) points(P$raw$x,P$raw$y,pch=".",...) else
             points(P$raw$x,P$raw$y,...)
@@ -919,9 +936,10 @@ plot.mgcv.smooth <- function(x,P=NULL,data=NULL,label="",se1.mult=1,se2.mult=2,
         if (scale==0&&is.null(ylim)) { 
           if (partial.resids) ylimit <- range(P$p.resid,na.rm=TRUE) else ylimit <-range(P$fit)
         }
-        if (!is.null(ylim)) ylimit <- ylim
+        ylimit <- if (is.null(ylim)) ylimit <- ylimit + shift else ylim
+        
         plot(P$x,trans(P$fit+shift),type="l",xlab=P$xlab,
-             ylab=P$ylab,ylim=trans(ylimit+shift),xlim=P$xlim,main=P$main,...)
+             ylab=P$ylab,ylim=trans(ylimit),xlim=P$xlim,main=P$main,...)
         if (rug) { 
           if (jit) rug(jitter(as.numeric(P$raw)),...)
           else rug(as.numeric(P$raw),...) 
@@ -938,10 +956,11 @@ plot.mgcv.smooth <- function(x,P=NULL,data=NULL,label="",se1.mult=1,se2.mult=2,
         if (scheme==1) { 
           persp(P$x,P$y,matrix(trans(P$fit+shift),n2,n2),xlab=P$xlab,ylab=P$ylab,
                           zlab=P$main,theta=theta,phi=phi,xlim=P$xlim,ylim=P$ylim,...)
-        } else if (scheme==2) {
+        } else if (scheme==2||scheme==3) {
+          if (scheme==3) hcolors <- grey(0:50/50)
           image(P$x,P$y,matrix(trans(P$fit+shift),n2,n2),xlab=P$xlab,ylab=P$ylab,
-                  main=P$main,xlim=P$xlim,ylim=P$ylim,col=heat.colors(50),...)
-          contour(P$x,P$y,matrix(trans(P$fit+shift),n2,n2),add=TRUE,col=3,...)
+                  main=P$main,xlim=P$xlim,ylim=P$ylim,col=hcolors,...)
+          contour(P$x,P$y,matrix(trans(P$fit+shift),n2,n2),add=TRUE,col=contour.col,...)
           if (rug) {  
             if (is.null(list(...)[["pch"]])) points(P$raw$x,P$raw$y,pch=".",...) else
             points(P$raw$x,P$raw$y,...)
@@ -966,7 +985,7 @@ plot.mgcv.smooth <- function(x,P=NULL,data=NULL,label="",se1.mult=1,se2.mult=2,
 plot.gam <- function(x,residuals=FALSE,rug=TRUE,se=TRUE,pages=0,select=NULL,scale=-1,n=100,n2=40,
                      pers=FALSE,theta=30,phi=30,jit=FALSE,xlab=NULL,ylab=NULL,main=NULL,
                      ylim=NULL,xlim=NULL,too.far=0.1,all.terms=FALSE,shade=FALSE,shade.col="gray80",
-                     shift=0,trans=I,seWithMean=FALSE,by.resids=FALSE,scheme=0,...)
+                     shift=0,trans=I,seWithMean=FALSE,unconditional=FALSE,by.resids=FALSE,scheme=0,...)
 
 # Create an appropriate plot for each smooth term of a GAM.....
 # x is a gam object
@@ -1004,6 +1023,11 @@ plot.gam <- function(x,residuals=FALSE,rug=TRUE,se=TRUE,pages=0,select=NULL,scal
   ## start of main function
   #########################
 
+  if (unconditional) {
+    if (is.null(x$Vc)) warning("Smoothness uncertainty corrected covariance not available") else 
+    x$Vp <- x$Vc ## cov matrix reset to full Bayesian
+  }
+
   w.resid<-NULL
   if (length(residuals)>1) # residuals supplied 
   { if (length(residuals)==length(x$residuals)) 
@@ -1021,7 +1045,8 @@ plot.gam <- function(x,residuals=FALSE,rug=TRUE,se=TRUE,pages=0,select=NULL,scal
     scheme <- rep(scheme[1],m)
   }
 
-  order <- attr(x$pterms,"order") # array giving order of each parametric term
+  ## array giving order of each parametric term...
+  order <- if (is.list(x$pterms))  unlist(lapply(x$pterms,attr,"order")) else attr(x$pterms,"order")
 
   if (all.terms) # plot parametric terms as well
   n.para <- sum(order==1) # plotable parametric terms   
@@ -1038,12 +1063,17 @@ plot.gam <- function(x,residuals=FALSE,rug=TRUE,se=TRUE,pages=0,select=NULL,scal
   }
 
   if (partial.resids) { ## getting information needed for partial residuals...
-    fv.terms <- predict(x,type="terms")
-    if (is.null(w.resid)) w.resid<-x$residuals*sqrt(x$weights) # weighted working residuals
+    if (is.null(w.resid)) { ## produce working resids if info available
+      if (is.null(x$residuals)||is.null(x$weights)) partial.resids <- FALSE else {
+        wr <- sqrt(x$weights)
+        w.resid <- x$residuals*wr/mean(wr) # weighted working residuals
+      }
+    }
+    if (partial.resids) fv.terms <- predict(x,type="terms") ## get individual smooth effects
   }
 
-  pd<-list(); ## plot data list
-  i<-1 # needs a value if no smooths, but parametric terms ...
+  pd <- list(); ## plot data list
+  i <- 1 # needs a value if no smooths, but parametric terms ...
 
   ##################################################
   ## First the loop to get the data for the plots...
@@ -1072,13 +1102,14 @@ plot.gam <- function(x,residuals=FALSE,rug=TRUE,se=TRUE,pages=0,select=NULL,scal
       if (se && P$se) { ## get standard errors for fit
         ## test whether mean variability to be added to variability (only for centred terms)
         if (seWithMean && attr(x$smooth[[i]],"nCons")>0) {
+          if (length(x$cmX) < ncol(x$Vp)) x$cmX <- c(x$cmX,rep(0,ncol(x$Vp)-length(x$cmX)))
           X1 <- matrix(x$cmX,nrow(P$X),ncol(x$Vp),byrow=TRUE)
           meanL1 <- x$smooth[[i]]$meanL1
           if (!is.null(meanL1)) X1 <- X1 / meanL1
           X1[,first:last] <- P$X
-          se.fit <- sqrt(rowSums((X1%*%x$Vp)*X1))
+          se.fit <- sqrt(pmax(0,rowSums((X1%*%x$Vp)*X1)))
         } else se.fit <- ## se in centred (or anyway unconstained) space only
-        sqrt(rowSums((P$X%*%x$Vp[first:last,first:last,drop=FALSE])*P$X))
+        sqrt(pmax(0,rowSums((P$X%*%x$Vp[first:last,first:last,drop=FALSE])*P$X)))
         if (!is.null(P$exclude)) P$se.fit[P$exclude] <- NA
       } ## standard errors for fit completed
       if (partial.resids) { P$p.resid <- fv.terms[,length(order)+i] + w.resid }
@@ -1141,7 +1172,7 @@ plot.gam <- function(x,residuals=FALSE,rug=TRUE,se=TRUE,pages=0,select=NULL,scal
   if (scale==-1&&is.null(ylim)) {
     k <- 0
     if (m>0) for (i in 1:m) if (pd[[i]]$plot.me&&pd[[i]]$scale) { ## loop through plot data 
-      if (se&&pd[[i]]$se) { ## require CIs on plots
+      if (se&&length(pd[[i]]$se)>1) { ## require CIs on plots
         ul<-pd[[i]]$fit+pd[[i]]$se
         ll<-pd[[i]]$fit-pd[[i]]$se
         if (k==0) { 
@@ -1165,6 +1196,7 @@ plot.gam <- function(x,residuals=FALSE,rug=TRUE,se=TRUE,pages=0,select=NULL,scal
         if (ll < ylim[1]) ylim[1] <- ll
       } ## partial resids done
     } ## loop end 
+    ylim <- trans(ylim+shift)
   } ## end of common scale computation
   
   ##############################################################
@@ -1187,9 +1219,11 @@ plot.gam <- function(x,residuals=FALSE,rug=TRUE,se=TRUE,pages=0,select=NULL,scal
   { class(x) <- c("gam","glm","lm") # needed to get termplot to call model.frame.glm 
     if (is.null(select)) {
       attr(x,"para.only") <- TRUE
-      termplot(x,se=se,rug=rug,col.se=1,col.term=1,...)
+      termplot(x,se=se,rug=rug,col.se=1,col.term=1,main=attr(x$pterms,"term.labels"),...)
     } else { # figure out which plot is required
       if (select > m) { 
+        ## can't figure out how to get this to work with more than first linear predictor
+        ## as termplots relies on matching terms to names in original data... 
         select <- select - m # i.e. which parametric term
         term.labels <- attr(x$pterms,"term.labels")
         term.labels <- term.labels[order==1]
@@ -1201,6 +1235,7 @@ plot.gam <- function(x,residuals=FALSE,rug=TRUE,se=TRUE,pages=0,select=NULL,scal
     }
   }
   if (pages>0) par(oldpar)
+  invisible(pd)
 } ## end plot.gam
 
 
@@ -1225,12 +1260,13 @@ exclude.too.far<-function(g1,g2,d1,d2,dist)
   if (m!=length(d2)) stop("data vectors are of different lengths")
   if (dist<0) stop("supplied dist negative")
   distance<-array(0,n)
-  o<-.C(C_MinimumSeparation,as.double(g1),as.double(g2),as.integer(n),as.double(d1),as.double(d2),
-         as.integer(m),distance=as.double(distance))  
-  res<-rep(FALSE,n)
+  o<-.C(C_MinimumSeparation,x=as.double(cbind(g1,g2)),n=as.integer(n), d=as.integer(2),
+                            t=as.double(cbind(d1,d2)),m=as.integer(m),distance=as.double(distance))
+ 
+  res <- rep(FALSE,n)
   res[o$distance > dist] <-TRUE
   res
-}
+} ## exclude.too.far
 
 
 
@@ -1281,8 +1317,9 @@ vis.gam <- function(x,view=NULL,cond=list(),n.grid=30,too.far=0,col=NA,color="he
     }
     if (k<2) stop("Model does not seem to have enough terms to do anything useful")
   } else { 
-    if (sum(view%in%v.names)!=2) stop(
-        paste(c("view variables must be one of",v.names),collapse=", "))
+    if (sum(view%in%v.names)!=2) stop(gettextf("view variables must be one of %s", 
+                                      paste(v.names, collapse = ", ")))
+
     for (i in 1:2) 
     if  (!inherits(x$var.summary[[view[i]]],c("numeric","factor"))) 
     stop("Don't know what to do with parametric terms that are not simple numeric or factor variables")
@@ -1294,27 +1331,11 @@ vis.gam <- function(x,view=NULL,cond=list(),n.grid=30,too.far=0,col=NA,color="he
   } else {
     if (length(unique(x$var.summary[[view[i]]]))<=1) ok <- FALSE 
   }
-  if (!ok) 
-  stop(paste("View variables must contain more than one value. view = c(",view[1],",",view[2],").",sep=""))
-
+  if (!ok) stop(gettextf("View variables must contain more than one value. view = c(%s,%s).",
+                view[1], view[2]))
+ 
   # now get the values of the variables which are not the arguments of the plotted surface
-#  marg<-x$model[1,]
-#  m.name<-names(x$model)
-#  for (i in 1:length(marg))
-#  { ma<-cond[[m.name[i]]]
-#    if (is.null(ma)) 
-#    { if (is.factor(x$model[[i]]))
-#      marg[[i]]<-factor(levels(x$model[[i]])[1],levels(x$model[[i]]))
-#      else if (para.term[i]&&is.matrix(x$model[[i]])) marg[[i]] <- t(colMeans(x$model[[i]]))
-#      else marg[[i]]<-mean(x$model[[i]]) 
-#    } else
-#    { if (is.factor(x$model[[i]]))
-#      marg[[i]]<-factor(ma,levels(x$model[[i]]))
-#      else marg[[i]]<-ma
-#    }
-#  }
-#  # marg includes conditioning values for view variables, but these will be ignored
-  
+
   # Make dataframe....
   if (is.factor(x$var.summary[[view[1]]]))
   m1<-fac.seq(x$var.summary[[view[1]]],n.grid)
@@ -1335,18 +1356,12 @@ vis.gam <- function(x,view=NULL,cond=list(),n.grid=30,too.far=0,col=NA,color="he
     else newd[[i]]<-rep(ma,n.grid*n.grid)
   }
   names(newd) <- v.names
-  #row.names <- attr(newd,"row.names")
-  #attributes(newd) <- attributes(x$model) # done so that handling of offsets etc. works
-  #attr(newd,"row.names") <- row.names
   newd[[view[1]]]<-v1
   newd[[view[2]]]<-v2
   # call predict.gam to get predictions.....
-  if (type=="link") zlab<-paste("linear predictor")
+  if (type=="link") zlab<-paste("linear predictor") ## ignore codetools
   else if (type=="response") zlab<-type
   else stop("type must be \"link\" or \"response\"")
-  ## turn newd into a model frame, so that names and averages are valid
-  #attributes(newd)<-attributes(x$model)
-  #attr(newd,"row.names")<-as.character(1:(n.grid*n.grid))
   fv <- predict.gam(x,newdata=newd,se.fit=TRUE,type=type)
   z <- fv$fit # store NA free copy now
   if (too.far>0) # exclude predictions too far from data
@@ -1408,7 +1423,7 @@ vis.gam <- function(x,view=NULL,cond=list(),n.grid=30,too.far=0,col=NA,color="he
     } else
     { stub <- paste(ifelse("xlab" %in% dnm, "" , ",xlab=view[1]"),
                     ifelse("ylab" %in% dnm, "" , ",ylab=view[2]"),
-                    ifelse("main" %in% dnm, "" , ",zlab=zlab"),",...)",sep="")
+                    ifelse("zlab" %in% dnm, "" , ",zlab=zlab"),",...)",sep="")
       if (color=="bw")
       { op <- par(bg="white")
         txt <- paste("persp(m1,m2,z,col=\"white\",zlim=c(min.z,max.z) ",stub,sep="") # assemble persp() call
@@ -1421,23 +1436,23 @@ vis.gam <- function(x,view=NULL,cond=list(),n.grid=30,too.far=0,col=NA,color="he
     }
   } else # add standard error surfaces
   { if (color=="bw"||color=="gray") 
-    { subs <- paste("grey are +/-",se,"s.e.") 
+    { subs <- paste("grey are +/-",se,"s.e.")  ## ignore codetools
       lo.col <- "gray" ## ignore codetools claims about this
       hi.col <- "gray" ## ignore codetools 
     } else
-    { subs<-paste("red/green are +/-",se,"s.e.")
+    { subs <- paste("red/green are +/-",se,"s.e.")
       lo.col <- "green"
       hi.col <- "red"
     }
-    if (!is.null(zlim))
-    { if (length(zlim)!=2||zlim[1]>=zlim[2]) stop("Something wrong with zlim")
+    if (!is.null(zlim)) {
+      if (length(zlim)!=2||zlim[1]>=zlim[2]) stop("Something wrong with zlim")
       min.z<-zlim[1]
       max.z<-zlim[2]
-    } else
-    { z.max<-max(fv$fit+fv$se.fit*se,na.rm=TRUE)
-      z.min<-min(fv$fit-fv$se.fit*se,na.rm=TRUE)
+    } else {
+      max.z<-max(fv$fit+fv$se.fit*se,na.rm=TRUE)
+      min.z<-min(fv$fit-fv$se.fit*se,na.rm=TRUE)
+      zlim<-c(min.z,max.z)
     }
-    zlim<-c(z.min,z.max)
     z<-fv$fit-fv$se.fit*se;z<-matrix(z,n.grid,n.grid)
     if (plot.type=="contour") warning("sorry no option for contouring with errors: try plot.gam")
 
@@ -1468,5 +1483,5 @@ vis.gam <- function(x,view=NULL,cond=list(),n.grid=30,too.far=0,col=NA,color="he
     eval(parse(text=txt))
 
   }
-}
+} ## vis.gam
 
