@@ -1,5 +1,5 @@
 #  File src/library/base/R/solve.R
-#  Part of the R package, http://www.R-project.org
+#  Part of the R package, https://www.R-project.org
 #
 #  Copyright (C) 1995-2013 The R Core Team
 #
@@ -14,7 +14,7 @@
 #  GNU General Public License for more details.
 #
 #  A copy of the GNU General Public License is available at
-#  http://www.r-project.org/Licenses/
+#  https://www.R-project.org/Licenses/
 
 solve.qr <- function(a, b, ...)
 {
@@ -51,8 +51,6 @@ solve.default <-
 	return(solve.qr(a, b, tol))
     }
 
-    if(LINPACK)
-        warning("LINPACK = TRUE is defunct and will be ignored", domain = NA)
     a <- as.matrix(a)
     if(missing(b)) {
         b <- diag(1.0, nrow(a))
