@@ -1,7 +1,7 @@
 #  File src/library/grid/R/frames.R
-#  Part of the R package, http://www.R-project.org
+#  Part of the R package, https://www.R-project.org
 #
-#  Copyright (C) 1995-2012 The R Core Team
+#  Copyright (C) 1995-2013 The R Core Team
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -14,7 +14,7 @@
 #  GNU General Public License for more details.
 #
 #  A copy of the GNU General Public License is available at
-#  http://www.r-project.org/Licenses/
+#  https://www.R-project.org/Licenses/
 
 ################
 # frame class
@@ -22,12 +22,8 @@
 # NOTE: make framevp separate slot (rather than combining with
 # normal vp slot) so that it can be edited (e.g., by grid.pack)
 frameGrob <- function(layout=NULL, name=NULL, gp=gpar(), vp=NULL) {
-  if (!is.null(layout))
-    framevp <- viewport(layout=layout)
-  else
-    framevp <- NULL
-  gTree(framevp=framevp, name=name, gp=gp, vp=vp,
-        cl="frame")
+  framevp <- if(!is.null(layout)) viewport(layout=layout) # else NULL
+  gTree(framevp=framevp, name=name, gp=gp, vp=vp, cl="frame")
 }
 
 # draw=TRUE will not draw anything, but will mean that
@@ -79,13 +75,13 @@ frameDim <- function(frame) {
 cellViewport <- function(col, row, border) {
   vp <- viewport(layout.pos.col=col, layout.pos.row=row)
   if (!is.null(border))
-    vp <- vpStack(vp,
-                  viewport(x=border[2L],
-                           y=border[1L],
-                           width=unit(1, "npc") - sum(border[c(2,4)]),
-                           height=unit(1, "npc") - sum(border[c(1,3)]),
-                           just=c("left", "bottom")))
-  vp
+    vpStack(vp,
+            viewport(x=border[2L],
+                     y=border[1L],
+                     width =unit(1, "npc") - sum(border[c(2,4)]),
+                     height=unit(1, "npc") - sum(border[c(1,3)]),
+                     just=c("left", "bottom")))
+  else vp
 }
 
 cellGrob <- function(col, row, border, grob, dynamic, vp) {
