@@ -62,7 +62,7 @@
 ##  anyLabel <- .sigLabel(anySig)
   newMethods <- names(newtable)
   for(what in newMethods) {
-    obj <- get(what, envir = newtable)
+    obj <- newtable[[what]]
     if(is.primitive(obj))
       sig <- anySig
     else if(is(obj, "MethodDefinition"))
@@ -73,7 +73,7 @@
            next # empty environment, ignore
        isDef <- vapply(objsWhat, is, logical(1L), "MethodDefinition")
        if (any(isDef)) {
-           sig <- tail(objsWhat[isDef], 1L)@defined
+           sig <- utils::tail(objsWhat[isDef], 1L)@defined
        } else {
            sig <- anySig
        }
@@ -103,7 +103,7 @@
         signames <- generic@signature
         length(signames) <- ns
         .resetTable(table, ns, signames)
-        assign(".SigLength", ns, envir = fenv)
+        fenv[[".SigLength"]] <- ns
         n <- ns
       }
     }
@@ -113,9 +113,9 @@
             ## must replace in .AllMTable also
             if(is.null(allTable))
                 allTable <- get(".AllMTable", envir = fenv)
-            assign(what, obj, envir = allTable)
+            allTable[[what]] <- obj
         }
-        assign(what, obj, envir = table)
+        table[[what]] <- obj
     }
     else if(exists(what, envir = table, inherits = FALSE) &&
             !all(obj@defined == "ANY") ) {
@@ -148,7 +148,7 @@
                 objw@defined <- objw@target <- sigw
                 remove(list = what, envir = obj)
                 var <- .pkgMethodLabel(objw)
-                if(nzchar(var)) assign(var, objw, envir = obj)
+                if(nzchar(var)) obj[[var]] <- objw
             }
         }
     }
@@ -743,9 +743,19 @@
     stop("Internal error in finding inherited methods; didn't return a unique method", domain = NA)
 }
 
-.findMethodInTable <- function(signature, table, fdef = NULL)
+.findMethodForFdef <- function(signature, table, fdef = NULL) {
+    value <- .findMethodInTable(signature, table, fdef)
+    if(is.null(value) && is(fdef, "genericFunction")) { # try without expanding signature
+        fullSig <- .matchSigLength(signature, fdef, environment(fdef), FALSE)
+        if(!identical(fullSig, signature))
+            value <- .findMethodInTable(signature, table, fdef, FALSE)
+    }
+    value
+}
+
+.findMethodInTable <- function(signature, table, fdef = NULL , expdSig = TRUE)
 {
-    if(is(fdef, "genericFunction"))
+    if(is(fdef, "genericFunction") && expdSig)
         signature <- .matchSigLength(signature, fdef, environment(fdef), FALSE)
     label <- .sigLabel(signature)
 ##     allMethods <- objects(table, all.names=TRUE)
