@@ -2,6 +2,7 @@
  *  S or S-PLUS or R.
  *
  *     Copyright (C) 1998 Douglas M. Bates and William N. Venables.
+ *     Copyright (C) 1999-2016 The R Core Team
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -15,7 +16,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, a copy is available at
- *  http://www.r-project.org/Licenses/
+ *  https://www.R-project.org/Licenses/
  *
  * The routines are loosely based on the pseudo-code in Schumacher (Wiley,
  * 1981) and the CMLIB library DBSPLINES.
@@ -23,6 +24,7 @@
 
 #include <R.h>
 #include <Rinternals.h>
+#include <string.h> // for memcpy
 
 #ifdef ENABLE_NLS
 #include <libintl.h>
@@ -88,17 +90,21 @@ diff_table(splPTR sp, double x, int ndiff)
 static void
 basis_funcs(splPTR sp, double x, double *b)
 {
-    int j, r;
-    double saved, term;
-
     diff_table(sp, x, sp->ordm1);
     b[0] = 1.;
-    for (j = 1; j <= sp->ordm1; j++) {
-	saved = 0.;
-	for (r = 0; r < j; r++) {
-	    term = b[r]/(sp->rdel[r] + sp->ldel[j - 1 - r]);
-	    b[r] = saved + sp->rdel[r] * term;
-	    saved = sp->ldel[j - 1 - r] * term;
+    for (int j = 1; j <= sp->ordm1; j++) {
+	double saved = 0.;
+	for (int r = 0; r < j; r++) { // do not divide by zero
+	    double den = sp->rdel[r] + sp->ldel[j - 1 - r];
+	    if(den != 0) {
+		double term = b[r]/den;
+		b[r] = saved + sp->rdel[r] * term;
+		saved = sp->ldel[j - 1 - r] * term;
+	    } else {
+		if(r != 0 || sp->rdel[r] != 0.)
+		    b[r] = saved;
+		saved = 0.;
+	    }
 	}
 	b[j] = saved;
     }
@@ -114,7 +120,7 @@ evaluate(splPTR sp, double x, int nder)
     if (sp->boundary && nder == sp->ordm1) { /* value is arbitrary */
 	return 0.0;
     }
-    while(nder--) {
+    while(nder--) {  // FIXME: divides by zero
 	for(inner = outer, apt = sp->a, lpt = ti - outer; inner--; apt++, lpt++)
 	    *apt = outer * (*(apt + 1) - *apt)/(*(lpt + outer) - *lpt);
 	outer--;
@@ -123,6 +129,7 @@ evaluate(splPTR sp, double x, int nder)
     while(outer--)
 	for(apt = sp->a, lpt = sp->ldel + outer, rpt = sp->rdel, inner = outer + 1;
 	    inner--; lpt--, rpt++, apt++)
+	    // FIXME: divides by zero
 	    *apt = (*(apt + 1) * *lpt + *apt * *rpt)/(*rpt + *lpt);
     return sp->a[0];
 }
@@ -134,41 +141,41 @@ spline_value(SEXP knots, SEXP coeff, SEXP order, SEXP x, SEXP deriv)
     SEXP val;
     splPTR sp;
     double *xx, *kk;
-    int der, i, n, nk;
+    int n, nk;
 
     PROTECT(knots = coerceVector(knots, REALSXP));
     kk = REAL(knots); nk = length(knots);
     PROTECT(coeff = coerceVector(coeff, REALSXP));
     PROTECT(x = coerceVector(x, REALSXP));
-    n = length(x);
-    xx = REAL(x);
-    PROTECT(order = coerceVector(order, INTSXP));
-    PROTECT(deriv = coerceVector(deriv, INTSXP));
-    der = INTEGER(deriv)[0];
-    PROTECT(val = allocVector(REALSXP, n));
+    xx = REAL(x); n = length(x);
+    int ord = asInteger(order);
+    int der = asInteger(deriv);
+    if (ord == NA_INTEGER || ord <= 0)
+	error(_("'ord' must be a positive integer"));
 
     /* populate the spl_struct */
-
     sp = (struct spl_struct *) R_alloc(1, sizeof(struct spl_struct));
-    sp->order = INTEGER(order)[0];
-    if (sp->order <= 0) { error(_("'ord' must be a positive integer")); }
-    sp->ordm1 = sp->order - 1;
+    sp->order = ord;
+    sp->ordm1 = ord - 1;
     sp->ldel = (double *) R_alloc(sp->ordm1, sizeof(double));
     sp->rdel = (double *) R_alloc(sp->ordm1, sizeof(double));
     sp->knots = kk; sp->nknots = nk;
     sp->coeff = REAL(coeff);
     sp->a = (double *) R_alloc(sp->order, sizeof(double));
 
-    for (i = 0; i < n; i++) {
+    PROTECT(val = allocVector(REALSXP, n));
+    double *rval = REAL(val);
+
+    for (int i = 0; i < n; i++) {
 	set_cursor(sp, xx[i]);
 	if (sp->curs < sp->order || sp->curs > (nk - sp->order)) {
-	    REAL(val)[i] = R_NaN;
+	    rval[i] = R_NaN;
 	} else {
-	    Memcpy(sp->a, REAL(coeff) + sp->curs - sp->order, sp->order);
-	    REAL(val)[i] = evaluate(sp, xx[i], der);
+	    Memcpy(sp->a, sp->coeff + sp->curs - sp->order, sp->order);
+	    rval[i] = evaluate(sp, xx[i], der);
 	}
     }
-    UNPROTECT(6);
+    UNPROTECT(4);
     return val;
 }
 
@@ -178,52 +185,48 @@ spline_basis(SEXP knots, SEXP order, SEXP xvals, SEXP derivs)
 {
 /* evaluate the non-zero B-spline basis functions (or their derivatives)
  * at xvals.  */
-    int nd, nk, nx, i, j, *ders;
-    double *kk, *xx;
-    SEXP val, offsets;
-    splPTR sp = (struct spl_struct *) R_alloc(1, sizeof(struct spl_struct));
 
     PROTECT(knots = coerceVector(knots, REALSXP));
-    kk = REAL(knots); nk = length(knots);
-    PROTECT(order = coerceVector(order, INTSXP));
+    double *kk = REAL(knots); int nk = length(knots);
+    int ord = asInteger(order);
     PROTECT(xvals = coerceVector(xvals, REALSXP));
-    xx = REAL(xvals); nx = length(xvals);
+    double *xx = REAL(xvals); int nx = length(xvals);
     PROTECT(derivs = coerceVector(derivs, INTSXP));
-    ders = INTEGER(derivs); nd = length(derivs);
+    int *ders = INTEGER(derivs), nd = length(derivs);
 
+    splPTR sp = (struct spl_struct *) R_alloc(1, sizeof(struct spl_struct));
     /* fill sp : */
-    sp->order = INTEGER(order)[0];
-    sp->ordm1 = sp->order - 1;
+    sp->order = ord;
+    sp->ordm1 = ord - 1;
     sp->rdel = (double *) R_alloc(sp->ordm1, sizeof(double));
     sp->ldel = (double *) R_alloc(sp->ordm1, sizeof(double));
     sp->knots = kk; sp->nknots = nk;
     sp->a = (double *) R_alloc(sp->order, sizeof(double));
-    PROTECT(val = allocMatrix(REALSXP, sp->order, nx));
-    PROTECT(offsets = allocVector(INTSXP, nx));
+    SEXP val = PROTECT(allocMatrix(REALSXP, sp->order, nx)),
+	offsets = PROTECT(allocVector(INTSXP, nx));
+    double *valM = REAL(val);
+    int *ioff = INTEGER(offsets);
 
-    for(i = 0; i < nx; i++) {
+    for(int i = 0; i < nx; i++) {
 	set_cursor(sp, xx[i]);
-	INTEGER(offsets)[i] = j = sp->curs - sp->order;
-	if (j < 0 || j > nk) {
-	    for (j = 0; j < sp->order; j++) {
-		REAL(val)[i * sp->order + j] = R_NaN;
+	int io = ioff[i] = sp->curs - sp->order;
+	if (io < 0 || io > nk) {
+	    for (int j = 0; j < sp->order; j++) {
+		valM[i * sp->order + j] = R_NaN;
 	    }
-	} else {
-	    if (ders[i % nd] > 0) { /* slow method for derivatives */
-		int ii;
-		for(ii = 0; ii < sp->order; ii++) {
-		    for(j = 0; j < sp->order; j++) sp->a[j] = 0;
-		    sp->a[ii] = 1;
-		    REAL(val)[i * sp->order + ii] =
-			evaluate(sp, xx[i], ders[i % nd]);
-		}
-	    } else {		/* fast method for value */
-		basis_funcs(sp, xx[i], REAL(val) + i * sp->order);
+	} else if (ders[i % nd] > 0) { /* slow method for derivatives */
+	    for(int ii = 0; ii < sp->order; ii++) {
+		for(int j = 0; j < sp->order; j++) sp->a[j] = 0;
+		sp->a[ii] = 1;
+		valM[i * sp->order + ii] =
+		    evaluate(sp, xx[i], ders[i % nd]);
 	    }
+	} else {		/* fast method for value */
+	    basis_funcs(sp, xx[i], valM + i * sp->order);
 	}
     }
     setAttrib(val, install("Offsets"), offsets);
-    UNPROTECT(6);
+    UNPROTECT(5);
     return val;
 }
 
@@ -237,9 +240,12 @@ const static R_CallMethodDef R_CallDef[] = {
 
 
 void
+#ifdef HAVE_VISIBILITY_ATTRIBUTE
+__attribute__ ((visibility ("default")))
+#endif
 R_init_splines(DllInfo *dll)
 {
     R_registerRoutines(dll, NULL, R_CallDef, NULL, NULL);
     R_useDynamicSymbols(dll, FALSE);
-//    R_forceSymbols(dll, TRUE); // too few to worry about, used in cobs*
+//    R_forceSymbols(dll, TRUE); // too few to worry about
 }
