@@ -1,7 +1,7 @@
 #  File src/library/stats/R/arima.R
-#  Part of the R package, http://www.R-project.org
+#  Part of the R package, https://www.R-project.org
 #
-#  Copyright (C) 2002-2014 The R Core Team
+#  Copyright (C) 2002-2015 The R Core Team
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -14,7 +14,7 @@
 #  GNU General Public License for more details.
 #
 #  A copy of the GNU General Public License is available at
-#  http://www.r-project.org/Licenses/
+#  https://www.R-project.org/Licenses/
 
 arima <- function(x, order = c(0L, 0L, 0L),
                   seasonal = list(order = c(0L, 0L, 0L), period = NA),
@@ -48,7 +48,7 @@ arima <- function(x, order = c(0L, 0L, 0L),
 
     arimaSS <- function(y, mod)
     {
-        ## next call changes mod elemnts a, P, Pn so beware!
+        ## next call changes mod components a, P, Pn so beware!
         .Call(C_ARIMA_Like, y, mod, 0L, TRUE)
     }
 
@@ -62,7 +62,7 @@ arima <- function(x, order = c(0L, 0L, 0L),
 				 error = function(e) NULL)))
 	    return(.Machine$double.xmax)# bad parameters giving error, e.g. in solve(.)
         if(ncxreg > 0) x <- x - xreg %*% par[narma + (1L:ncxreg)]
-        ## next call changes objects a, P, Pn so beware!
+        ## next call changes Z components a, P, Pn so beware!
         res <- .Call(C_ARIMA_Like, x, Z, 0L, FALSE)
         s2 <- res[1L]/res[3L]
         0.5*(log(s2) + res[2L]/res[3L])
@@ -170,8 +170,7 @@ arima <- function(x, order = c(0L, 0L, 0L),
     if (method == "CSS" || method == "CSS-ML") {
         ncond <- order[2L] + seasonal$order[2L] * seasonal$period
         ncond1 <- order[1L] + seasonal$period * seasonal$order[1L]
-        ncond <- if (!missing(n.cond)) ncond + max(n.cond, ncond1)
-        else ncond + ncond1
+	ncond <- ncond + if(!missing(n.cond)) max(n.cond, ncond1) else ncond1
     } else ncond <- 0
 
     if (is.null(fixed)) fixed <- rep(NA_real_, narma + ncxreg)
@@ -213,7 +212,8 @@ arima <- function(x, order = c(0L, 0L, 0L),
             ## Degenerate model. Proceed anyway so as not to break old code
             fit <- lm(x ~ xreg - 1, na.action = na.omit)
         }
-        n.used <- sum(!is.na(resid(fit))) - length(Delta)
+        isna <- is.na(x) | apply(xreg, 1L, anyNA)
+        n.used <- sum(!isna) - length(Delta)
         init0 <- c(init0, coef(fit))
         ses <- summary(fit)$coefficients[, 2L]
         parscale <- c(parscale, 10 * ses)
@@ -295,7 +295,7 @@ arima <- function(x, order = c(0L, 0L, 0L),
             list(convergence = 0, par = numeric(),
                  value = armafn(numeric(), as.logical(transform.pars)))
         else
-            optim(init[mask], armafn,  method = optim.method,
+            optim(init[mask], armafn, method = optim.method,
                   hessian = TRUE, control = optim.control,
                   trans = as.logical(transform.pars))
         if(res$convergence > 0)
@@ -364,7 +364,8 @@ arima <- function(x, order = c(0L, 0L, 0L),
     structure(list(coef = coef, sigma2 = sigma2, var.coef = var, mask = mask,
 		   loglik = -0.5 * value, aic = aic, arma = arma,
 		   residuals = resid, call = match.call(), series = series,
-		   code = res$convergence, n.cond = ncond, model = mod),
+		   code = res$convergence, n.cond = ncond, nobs = n.used,
+		   model = mod),
 	      class = "Arima")
 }
 
@@ -442,9 +443,9 @@ predict.Arima <-
         se <- ts(sqrt(z[[2L]] * object$sigma2),
                  start = xtsp[2L] + deltat(rsd),
                  frequency = xtsp[3L])
-        return(list(pred=pred, se=se))
+        list(pred=pred, se=se)
     }
-    else return(pred)
+    else pred
 }
 
 
@@ -494,7 +495,7 @@ vcov.Arima <- function (object, ...) object$var.coef
 
 logLik.Arima <- function (object, ...) {
     res <- if(is.na(object$aic)) NA
-    else structure(object$loglik, df=sum(object$mask) + 1)
+    else structure(object$loglik, df = sum(object$mask) + 1, nobs = object$nobs)
     class(res) <- "logLik"
     res
 }
