@@ -1,7 +1,7 @@
 #  File src/library/tools/R/Rd2txt.R
-#  Part of the R package, http://www.R-project.org
+#  Part of the R package, https://www.R-project.org
 #
-#  Copyright (C) 1995-2012 The R Core Team
+#  Copyright (C) 1995-2015 The R Core Team
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -14,7 +14,7 @@
 #  GNU General Public License for more details.
 #
 #  A copy of the GNU General Public License is available at
-#  http://www.r-project.org/Licenses/
+#  https://www.R-project.org/Licenses/
 
 ## This stops on
 ##  unrecognized tag
@@ -144,7 +144,7 @@ transformMethod <- function(i, blocks, Rdfile) {
     }
 
     rewriteBlocks <- function()
-    	c(blocks[seq_len(j-1)],
+    	c(blocks[seq_len(j-1L)],
     	            editblock(blocks[[j]],
     	                      paste(chars[seq_len(char)], collapse="")),
     	            if (char < length(chars))
@@ -175,7 +175,7 @@ transformMethod <- function(i, blocks, Rdfile) {
 	## need to assemble the call by matching parens in RCODE
 	findOpen(i) # Sets chars, char and j
 	chars[char] <- ""
-	blocks <- c(blocks[seq_len(j-1)],
+	blocks <- c(blocks[seq_len(j-1L)],
 	            editblock(blocks[[j]],
 	                      paste(chars[seq_len(char)], collapse="")),
 	            if (char < length(chars))
@@ -209,16 +209,20 @@ transformMethod <- function(i, blocks, Rdfile) {
 		   generic)) {
         ## Binary operators and unary '!'.
 	findOpen(i)
-	chars[char] <- ""
-	blocks <- rewriteBlocks()
 
 	if (generic != "!") {
+            chars[char] <- ""
+            blocks <- rewriteBlocks()
 	    findComma(j)
 	    chars[char] <- paste0(" ", generic, " ")
 	    # Delete blanks after the comma
 	    deleteBlanks()
 	    blocks <- rewriteBlocks()
-	}
+	} else {
+            chars[char] <- "!"
+	    blocks <- rewriteBlocks()
+        }
+
 	findClose(j)
 	chars[char] <- ""
 	blocks[j] <- editblock(blocks[[j]],
@@ -233,14 +237,31 @@ transformMethod <- function(i, blocks, Rdfile) {
 	methodtype <- if (grepl("<-", blocks[[j]])) "replacement " else ""
     }
 
-    if (blocktag == "\\S4method")
-    	blocks <- c( blocks[seq_len(i-1)],
-		     list(structure(paste0("## S4 ", methodtype, "method for signature '"),
-			   Rd_tag="RCODE", srcref=srcref)),
-		     class,
-		     list(structure("'\n", Rd_tag="RCODE", srcref=srcref)),
-		     blocks[-seq_len(i)] )
-    else if (default)
+    if (blocktag == "\\S4method") {
+        ## some signatures are very long.
+        blocks <- if(nchar(class) > 50L) {
+            cl <- paste0("'", as.character(class), "'")
+            if(nchar(cl) > 70L) {
+                cl <- strsplit(cl, ",")[[1L]]
+                ncl <- length(cl)
+                cl[-ncl] <- paste0(cl[-ncl], ",")
+                cl[-1L] <- paste0("  ", cl[-1L])
+            }
+            cl <- paste("##", cl, collapse="\n")
+            c( blocks[seq_len(i-1L)],
+              list(structure(paste0("## S4 ", methodtype, "method for signature \n"),
+                             Rd_tag="RCODE", srcref=srcref)),
+              list(structure(cl, Rd_tag="TEXT", srcref=srcref)),
+              list(structure("\n", Rd_tag="RCODE", srcref=srcref)),
+              blocks[-seq_len(i)] )
+        } else
+            c( blocks[seq_len(i-1L)],
+              list(structure(paste0("## S4 ", methodtype, "method for signature '"),
+                             Rd_tag="RCODE", srcref=srcref)),
+              class,
+              list(structure("'\n", Rd_tag="RCODE", srcref=srcref)),
+              blocks[-seq_len(i)] )
+    } else if (default)
     	blocks <- c( blocks[seq_len(i-1)],
     		     list(structure(paste0("## Default S3 ", methodtype, "method:\n"),
     		     	       Rd_tag="RCODE", srcref=srcref)),
@@ -341,7 +362,7 @@ Rd2txt <-
                 left <- excess %/% 2
                 right <- excess-left
             }
-            paste(c(rep(" ", left), x, rep(" ", right)), collapse = "")
+            paste(c(rep_len(" ", left), x, rep_len(" ", right)), collapse = "")
         } else x
     }
 
@@ -375,9 +396,6 @@ Rd2txt <-
         linestart <<- trail
     }
 
-    blanks <- function(n)
-	if (n) paste(rep(" ", n), collapse="") else ""
-
     flushBuffer <- function() {
     	if (!length(buffer)) return()
 
@@ -413,10 +431,10 @@ Rd2txt <-
 	} else {  # Not wrapping
 	    if (keepFirstIndent) {
 		if (length(buffer) > 1L)
-		    buffer[-1L] <<- paste0(blanks(indent), buffer[-1L])
+		    buffer[-1L] <<- paste0(strrep(" ", indent), buffer[-1L])
 		keepFirstIndent <- FALSE
 	    } else
-		buffer <<- paste0(blanks(indent), buffer)
+		buffer <<- paste0(strrep(" ", indent), buffer)
 	}
 
     	if (length(buffer)) WriteLines(buffer, con, outputEncoding)
@@ -452,13 +470,6 @@ Rd2txt <-
         psub1("\\s*$", "", x)
     }
 
-    striptitle <- function(text) {
-        text <- fsub("\\", "", text)
-        text <- fsub("---", "_", text)
-        text <- fsub("--", "-", text)
-        text
-    }
-
     ## underline via backspacing
     txt_header <- function(header) {
         opts <- Rd2txt_options()
@@ -492,7 +503,7 @@ Rd2txt <-
     	    buffer <<- buffer[-length(buffer)]
 	flushBuffer()
 	if (n > haveBlanks) {
-	    buffer <<- rep("", n - haveBlanks)
+	    buffer <<- rep_len("", n - haveBlanks)
 	    flushBuffer()
 	    haveBlanks <<- n
 	}
@@ -585,18 +596,28 @@ Rd2txt <-
                        writeQ(block, tag, quote="\\sQuote")
                    else writeContent(block,tag)
                },
-               "\\email"=  put("<email: ",
-                               gsub("\n", "", paste(as.character(block), collapse="")),
-                               ">"),
-               "\\url"= put("<URL: ",
-                              gsub("\n", "", paste(as.character(block), collapse="")),
-                              ">") ,
-               "\\href"= {
+               "\\email" = {
+                   put("<email: ",
+                       trimws(gsub("\n", "",
+                                   paste(as.character(block),
+                                         collapse=""))),
+                       ">")
+               },
+               "\\url" = {
+                   put("<URL: ",
+                       trimws(gsub("\n", "",
+                                   paste(as.character(block),
+                                         collapse=""))),
+                       ">")
+               },
+               "\\href" = {
                    opts <- Rd2txt_options()
                    writeContent(block[[2L]], tag)
                    if (opts$showURLs)
   			put(" (URL: ",
-  			    gsub("\n", "", paste(as.character(block[[1L]]), collapse="")),
+  			    trimws(gsub("\n", "",
+                                        paste(as.character(block[[1L]]),
+                                              collapse=""))),
   			    ")")
                },
                "\\Sexpr"= put(as.character.Rd(block, deparse=TRUE)),
@@ -736,8 +757,8 @@ Rd2txt <-
                     })
         rows <- entries[[length(entries)]]$row
         cols <- max(sapply(entries, function(e) e$col))
-        widths <- rep(0L, cols)
-        lines <- rep(1L, rows)
+        widths <- rep_len(0L, cols)
+        lines <- rep_len(1L, rows)
         for (i in seq_along(entries)) {
             e <- entries[[i]]
             while(length(e$text) && !nzchar(e$text[length(e$text)])) {
@@ -750,7 +771,7 @@ Rd2txt <-
         }
         result <- matrix("", sum(lines), cols)
         for (i in seq_len(cols))
-            result[, i] <- blanks(widths[i])
+            result[, i] <- strrep(" ", widths[i])
         firstline <- c(1L, 1L+cumsum(lines))
         for (i in seq_along(entries)) {
             e <- entries[[i]]
@@ -835,7 +856,7 @@ Rd2txt <-
                                   indent <<- max(opts$minIndent,
                                                  indent + opts$extraIndent)
                                   keepFirstIndent <<- TRUE
-                                  putw(paste(rep(" ", indent0), collapse=""),
+                                  putw(strrep(" ", indent0),
                                        frmt(paste0(DLlab),
                                             justify="left", width=indent),
                                        " ")
@@ -894,7 +915,7 @@ Rd2txt <-
             return()
     	save <- c(indent, sectionLevel, keepFirstIndent, dropBlank, wrapping)
     	blankLine(min(sectionLevel, 1L))
-    	titlePrefix <- paste(rep("  ", sectionLevel), collapse="")
+    	titlePrefix <- strrep("  ", sectionLevel)
     	opts <- Rd2txt_options()
         indent <<- opts$sectionIndent + opts$sectionExtra*sectionLevel
         sectionLevel <<- sectionLevel + 1
@@ -908,7 +929,7 @@ Rd2txt <-
             wrapping <<- TRUE
             keepFirstIndent <<- FALSE
     	    writeContent(section[[2L]], tag)
-    	} else if (tag %in% c("\\usage", "\\synopsis", "\\examples")) {
+    	} else if (tag %in% c("\\usage", "\\examples")) {
             putf(txt_header(sectionTitles[tag]), ":")
             blankLine()
             dropBlank <<- TRUE
@@ -967,8 +988,8 @@ Rd2txt <-
 		right <- paste0(right, "(", encoding, ")")
 	    pad <- max(HDR_WIDTH - nchar(left, "w") - nchar(mid, "w") - nchar(right, "w"), 0)
 	    pad0 <- pad %/% 2L
-	    pad1 <- paste(rep.int(" ", pad0), collapse = "")
-	    pad2 <- paste(rep.int(" ", pad - pad0), collapse = "")
+	    pad1 <- strrep(" ", pad0)
+	    pad2 <- strrep(" ", pad - pad0)
 	    putf(paste0(left, pad1, mid, pad2, right, "\n\n"))
 	}
 
