@@ -67,7 +67,7 @@
 /*
  *  R : A Computer Langage for Statistical Data Analysis
  *  Copyright (C) 1995, 1996, 1997  Robert Gentleman and Ross Ihaka
- *  Copyright (C) 1997--2013  The R Core Team
+ *  Copyright (C) 1997--2015  The R Core Team
  *  Copyright (C) 2010 Duncan Murdoch
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -2211,7 +2211,7 @@ static int xxungetc(int c)
     R_ParseContextLine = parseState.xxlineno;
     
     R_ParseContext[R_ParseContextLast] = '\0';
-    /* Mac OS X requires us to keep this non-negative */
+    /* macOS requires us to keep this non-negative */
     R_ParseContextLast = (R_ParseContextLast + PARSE_CONTEXT_SIZE - 1) 
 	% PARSE_CONTEXT_SIZE;
     if(npush >= PUSHBACK_BUFSIZE - 2) return R_EOF;
@@ -2478,7 +2478,7 @@ static void yyerror(const char *s)
     	snprintf(ParseErrorMsg, PARSE_ERROR_SIZE,"%s", s);
     }
     filename = findVar(install("filename"), parseState.SrcFile);
-    if (isString(filename) && length(filename))
+    if (isString(filename) && LENGTH(filename))
     	strncpy(ParseErrorFilename, CHAR(STRING_ELT(filename, 0)), PARSE_ERROR_SIZE - 1);
     else
         ParseErrorFilename[0] = '\0';
@@ -2709,7 +2709,7 @@ static void PopState() {
 
 /* "do_parseLatex" 
 
- .External2(CC_parseLatex, file, srcfile, verbose, basename, warningCalls)
+ .External2(C_parseLatex, file, srcfile, verbose, basename, warningCalls)
  If there is text then that is read and the other arguments are ignored.
 */
 
