@@ -1,7 +1,7 @@
 #  File src/library/utils/R/packages2.R
 #  Part of the R package, https://www.R-project.org
 #
-#  Copyright (C) 1995-2015 The R Core Team
+#  Copyright (C) 1995-2016 The R Core Team
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -19,6 +19,12 @@
 if (.Platform$OS.type == "windows")
     .install.macbinary <- function(...) NULL	# globalVariables isn't available, so use this to suppress the warning
 
+isBasePkg <- function(pkg) {
+  priority <- tryCatch(packageDescription(pkg, fields = "Priority"),
+                       error = function(e) e, warning = function(e) e)
+  identical(priority, "base")
+}
+
 getDependencies <-
     function(pkgs, dependencies = NA, available = NULL, lib = .libPaths()[1L],
              binary = FALSE)
@@ -53,6 +59,13 @@ getDependencies <-
 				 "packages %s are not available (%s)"),
 			paste(sQuote(p0[miss]), collapse = ", "), msg),
                 domain = NA, call. = FALSE)
+        base <- vapply(p0[miss], isBasePkg, FALSE)
+        if (sum(base))
+          warning(sprintf(ngettext(sum(base),
+                                   "package %s is a base package, and should not be updated",
+                                   "packages %s are base packages, and should not be updated"),
+                          paste(sQuote(p0[miss][base]), collapse = ", ")),
+                  domain = NA, call. = FALSE)
         if (sum(miss) == 1L &&
             !is.na(w <- match(tolower(p0[miss]),
                               tolower(row.names(available))))) {
@@ -130,7 +143,7 @@ install.packages <-
         if (type2 == "source")
             stop("type 'binary' is not supported on this platform")
         else type <- type2
-        if(!missing(contriburl) || !is.null(available))
+        if(type == "both" && (!missing(contriburl) || !is.null(available)))
            stop("specifying 'contriburl' or 'available' requires a single type, not type = \"both\"")
     }
     if (is.logical(clean) && clean)
@@ -206,7 +219,7 @@ install.packages <-
         ## if no packages were specified, use a menu
 	if(.Platform$OS.type == "windows" || .Platform$GUI == "AQUA"
            || (capabilities("tcltk")
-               && capabilities("X11") && suppressWarnings(tcltk:::.TkUp)) ) {
+               && capabilities("X11") && suppressWarnings(tcltk::.TkUp)) ) {
             ## this is the condition for a graphical select.list()
 	} else
 	    stop("no packages were specified")
@@ -217,7 +230,7 @@ install.packages <-
 	    av <- available.packages(contriburl = contriburl, method = method)
 	    if (missing(repos)) ## Evaluating contriburl may have changed repos, which may be used below
 	      repos <- getOption("repos")
-            if(type != "both") available <- av 
+            if(type != "both") available <- av
         } else av <- available
 	if(NROW(av)) {
             ## avoid duplicate entries in menus, since the latest available
@@ -318,7 +331,7 @@ install.packages <-
            }
         }
     }
-    
+
     ## check if we should infer the type
     if (length(pkgs) == 1L && is.null(repos) && type == "both") {
     	if (  (type2 %in% "win.binary" && grepl("[.]zip$", pkgs))
@@ -358,12 +371,12 @@ install.packages <-
 
 
     ## Look at type == "both"
-    ## NB it is only safe to use binary packages with a Mac OS X
+    ## NB it is only safe to use binary packages with a macOS
     ## build that uses the same R foundation layout as CRAN since
     ## paths in DSOs are hard-coded.
     if (type == "both") {
         if (type2 == "source")
-            stop("type == \"both\" can only be used on Windows or a CRAN build for Mac OS X")
+            stop("type == \"both\" can only be used on Windows or a CRAN build for macOS")
         if (!missing(contriburl) || !is.null(available)) type <- type2
     }
 
@@ -526,7 +539,7 @@ install.packages <-
 
     if(.Platform$OS.type == "windows") {
         if(substr(type, 1L, 10L) == "mac.binary")
-            stop("cannot install MacOS X binary packages on Windows")
+            stop("cannot install macOS binary packages on Windows")
 
         if(type %in% "win.binary") {
             ## include local .zip files
@@ -552,7 +565,7 @@ install.packages <-
     } else {
         if(substr(type, 1L, 10L) == "mac.binary") {
             if(!grepl("darwin", R.version$platform))
-                stop("cannot install MacOS X binary packages on this platform")
+                stop("cannot install macOS binary packages on this platform")
             .install.macbinary(pkgs = pkgs, lib = lib, contriburl = contriburl,
                                method = method, available = available,
                                destdir = destdir,
@@ -735,7 +748,7 @@ install.packages <-
                 ##   cmd <- paste(c(shQuote(command), env, args),
                 ##                collapse = " ")
                 ## on Windows?
-                cmd <- paste(c(shQuote(cmd0), args), collapse = " ")
+                cmd <- paste(c("MAKEFLAGS=", shQuote(cmd0), args), collapse = " ")
                 ## </NOTE>
                 deps <- aDL[[pkg]]
                 deps <- deps[deps %in% upkgs]
@@ -799,10 +812,11 @@ install.packages <-
                 file.remove(outfiles)
             }
         }
+        ## Using stderr is the wish of PR#16420
         if(!quiet && nonlocalrepos && !is.null(tmpd) && is.null(destdir))
             cat("\n", gettextf("The downloaded source packages are in\n\t%s",
                                sQuote(normalizePath(tmpd, mustWork = FALSE))),
-                "\n", sep = "")
+                "\n", sep = "", file = stderr())
         ## update packages.html on Unix only if .Library was installed into
         libs_used <- unique(update[, 2L])
         if(.Platform$OS.type == "unix" && .Library %in% libs_used) {
@@ -859,10 +873,9 @@ registerNames <- function(names, package, .listFile, add = TRUE) {
 packageName <- function(env = parent.frame()) {
     if (!is.environment(env)) stop("'env' must be an environment")
     env <- topenv(env)
-    if (exists(".packageName", envir = env, inherits = FALSE))
-	get(".packageName", envir = env, inherits = FALSE)
+    if (!is.null(pn <- get0(".packageName", envir = env, inherits = FALSE)))
+	pn
     else if (identical(env, .BaseNamespaceEnv))
 	"base"
-    else
-	NULL
+    ## else NULL
 }
