@@ -1,7 +1,7 @@
 #  File src/library/utils/R/sessionInfo.R
 #  Part of the R package, https://www.R-project.org
 #
-#  Copyright (C) 1995-2014 The R Core Team
+#  Copyright (C) 1995-2016 The R Core Team
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -36,10 +36,10 @@ sessionInfo <- function(package = NULL)
                    "Linux" = if(file.exists("/etc/os-release")) {
     ## http://www.freedesktop.org/software/systemd/man/os-release.html
                        tmp <- readLines("/etc/os-release")
-                       t2 <- if (any(grepl("^PRETTY_NAME=", tmp)))
+                       t2 <- if (any(startsWith(tmp, "PRETTY_NAME=")))
                            sub("^PRETTY_NAME=", "",
                                grep("^PRETTY_NAME=", tmp, value = TRUE)[1L])
-                       else if (any(grepl("^NAME", tmp)))
+                       else if (any(startsWith(tmp, "NAME")))
                            ## could check for VERSION or VERSION_ID
                            sub("^NAME=", "",
                                grep("^NAME=", tmp, value = TRUE)[1L])
@@ -56,7 +56,8 @@ sessionInfo <- function(package = NULL)
                        ver <- sub(".*<string>", "", ver)
                        ver <- sub("</string>$", "", ver)
                        ver1 <- strsplit(ver, ".", fixed = TRUE)[[1L]][2L]
-                       sprintf("OS X %s (%s)", ver,
+                       sprintf("%s %s %s",
+                               ifelse(as.numeric(ver1) < 12, "OS X", "macOS"),
                                switch(ver1,
                                       "4" = "Tiger",
                                       "5" = "Leopard",
@@ -66,7 +67,8 @@ sessionInfo <- function(package = NULL)
                                       "9" = "Mavericks",
                                       "10" = "Yosemite",
                                       "11" = "El Capitan",
-                                      "unknown"))
+                                      "12" = "Sierra",
+                                      ""), ver)
                    },
                    "SunOS" = {
                        ver <- system('uname -r', intern = TRUE)
@@ -79,8 +81,9 @@ sessionInfo <- function(package = NULL)
     if(is.null(package)){
         package <- grep("^package:", search(), value=TRUE)
         # weed out environments which are not really packages
-        keep <- sapply(package, function(x) x == "package:base" || !is.null(attr(as.environment(x), "path")))
-        package <- sub("^package:", "", package[keep])
+        keep <- vapply(package, function(x) x == "package:base"
+                       || !is.null(attr(as.environment(x), "path")), NA)
+        package <- .rmpkg(package[keep])
     }
 
     ## no need to re-encode given what we extract.
@@ -105,7 +108,7 @@ sessionInfo <- function(package = NULL)
     z
 }
 
-print.sessionInfo <- function(x, locale=TRUE, ...)
+print.sessionInfo <- function(x, locale = TRUE, ...)
 {
     mkLabel <- function(L, n) {
         vers <- sapply(L[[n]], function(x) x[["Version"]])
@@ -117,7 +120,7 @@ print.sessionInfo <- function(x, locale=TRUE, ...)
     cat("Platform: ", x$platform, "\n", sep = "")
     if (!is.null(x$running)) cat("Running under: ",  x$running, "\n", sep = "")
     cat("\n")
-    if(locale){
+    if(locale) {
         cat("locale:\n")
 	print(strsplit(x$locale, ";", fixed=TRUE)[[1]], quote=FALSE, ...)
         cat("\n")
@@ -126,16 +129,16 @@ print.sessionInfo <- function(x, locale=TRUE, ...)
     print(x$basePkgs, quote=FALSE, ...)
     if(!is.null(x$otherPkgs)){
         cat("\nother attached packages:\n")
-	print(mkLabel(x, "otherPkgs"), quote=FALSE, ...)
+	print(mkLabel(x, "otherPkgs"), quote = FALSE, ...)
     }
     if(!is.null(x$loadedOnly)){
         cat("\nloaded via a namespace (and not attached):\n")
-	print(mkLabel(x, "loadedOnly"), quote=FALSE, ...)
+	print(mkLabel(x, "loadedOnly"), quote = FALSE, ...)
     }
     invisible(x)
 }
 
-toLatex.sessionInfo <- function(object, locale=TRUE, ...)
+toLatex.sessionInfo <- function(object, locale = TRUE, ...)
 {
     opkgver <- sapply(object$otherPkgs, function(x) x$Version)
     nspkgver <- sapply(object$loadedOnly, function(x) x$Version)
@@ -143,7 +146,7 @@ toLatex.sessionInfo <- function(object, locale=TRUE, ...)
            paste0("  \\item ", object$R.version$version.string,
                   ", \\verb|", object$R.version$platform, "|"))
 
-    if(locale){
+    if(locale) {
         z <- c(z,
                paste0("  \\item Locale: \\verb|",
                       gsub(";","|, \\\\verb|", object$locale) , "|"))
