@@ -1,5 +1,7 @@
 #  File src/library/utils/R/toLatex.R
-#  Part of the R package, http://www.R-project.org
+#  Part of the R package, https://www.R-project.org
+#
+#  Copyright (C) 1995-2013 The R Core Team
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -12,20 +14,14 @@
 #  GNU General Public License for more details.
 #
 #  A copy of the GNU General Public License is available at
-#  http://www.r-project.org/Licenses/
+#  https://www.R-project.org/Licenses/
 
 toBibtex <- function(object, ...) UseMethod("toBibtex")
 
-print.Bibtex <- function(x, prefix="", ...)
-{
-    writeLines(paste0(prefix, unclass(x)))
-    invisible(x)
-}
-
 toLatex <- function(object, ...) UseMethod("toLatex")
 
-print.Latex <- function(x, prefix="", ...)
+print.Bibtex <- print.Latex <- function(x, prefix = "", ...)
 {
-    writeLines(paste0(prefix, unclass(x)))
+    writeLines(paste0(prefix, unclass(x)), ...)
     invisible(x)
 }
