@@ -128,6 +128,10 @@ R_CPFun CPFun[MAX_CACHE];
 int nCPFun = 0;
 #endif
 
+/* Note that it is likely that dlopen will use up at least one file
+   descriptor for each DLL loaded (it may load further dynamically
+   linked libraries), so we do not want to get close to the fd limit
+   (which may be as low as 256). */
 #define MAX_NUM_DLLS	100
 
 static int CountDLL = 0;
@@ -976,10 +980,10 @@ Rf_MakeRegisteredNativeSymbol(R_RegisteredNativeSymbol *symbol)
     R_RegisteredNativeSymbol *copy;
     copy = (R_RegisteredNativeSymbol *) malloc(1 * sizeof(R_RegisteredNativeSymbol));
     if(!copy) {
-        error(ngettext("cannot allocate memory for registered native symbol (%d byte)",
+	error(ngettext("cannot allocate memory for registered native symbol (%d byte)",
 		       "cannot allocate memory for registered native symbol (%d bytes)",
-                      (int) sizeof(R_RegisteredNativeSymbol)),
-                      (int) sizeof(R_RegisteredNativeSymbol));
+		      (int) sizeof(R_RegisteredNativeSymbol)),
+		      (int) sizeof(R_RegisteredNativeSymbol));
     }
     *copy = *symbol;
 
@@ -1169,7 +1173,7 @@ createRSymbolObject(SEXP sname, DL_FUNC f, R_RegisteredNativeSymbol *symbol,
 
     PROTECT(klass = allocVector(STRSXP, (symbol->type != R_ANY_SYM ? 2 : 1)));
     numProtects++;
-    SET_STRING_ELT(klass, length(klass)-1, mkChar("NativeSymbolInfo"));
+    SET_STRING_ELT(klass, LENGTH(klass) - 1, mkChar("NativeSymbolInfo"));
 
     if(n > 3) {
 	/* Add the registration information:
