@@ -1,7 +1,7 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
  *  Copyright (C) 1995, 1996  Robert Gentleman and Ross Ihaka
- *  Copyright (C) 1997--2015  The R Core Team
+ *  Copyright (C) 1997--2016  The R Core Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Pulic License as published by
@@ -72,10 +72,8 @@ abbreviate chartr make.names strtrim tolower toupper give error.
 #include <Defn.h>
 #include <Internal.h>
 #include <errno.h>
-
 #include <R_ext/RS.h>  /* for Calloc/Free */
-#include <Rinternals.h> // R_nchar()
-
+#include <R_ext/Itermacros.h>
 #include <rlocale.h>
 
 /* We use a shared buffer here to avoid reallocing small buffers, and
@@ -133,6 +131,7 @@ SEXP attribute_hidden do_nzchar(SEXP call, SEXP op, SEXP args, SEXP env)
     return ans;
 }
 
+/* R strings are limited to 2^31 - 1 bytes on all platforms */
 int R_nchar(SEXP string, nchar_type type_,
 	    Rboolean allowNA, Rboolean keepNA, const char* msg_name)
 {
@@ -251,7 +250,7 @@ SEXP attribute_hidden do_nchar(SEXP call, SEXP op, SEXP args, SEXP env)
 	keepNA = asLogical(CADDDR(args));
 	if (keepNA == NA_LOGICAL) // default
 	    keepNA = (type_ == Width) ? FALSE : TRUE;
-    } else  keepNA = FALSE; // default
+    } else  keepNA = (type_ == Width) ? FALSE : TRUE;
     PROTECT(s = allocVector(INTSXP, len));
     int *s_ = INTEGER(s);
     for (R_xlen_t i = 0; i < len; i++) {
@@ -300,43 +299,37 @@ static void substr(char *buf, const char *str, int ienc, int sa, int so)
     *buf = '\0';
 }
 
-SEXP attribute_hidden do_substr(SEXP call, SEXP op, SEXP args, SEXP env)
+SEXP attribute_hidden
+do_substr(SEXP call, SEXP op, SEXP args, SEXP env)
 {
-    SEXP s, x, sa, so, el;
-    R_xlen_t i, len;
-    int start, stop, k, l;
-    size_t slen;
-    cetype_t ienc;
-    const char *ss;
-    char *buf;
-
+    SEXP s, x;
     checkArity(op, args);
     x = CAR(args);
-    sa = CADR(args);
-    so = CADDR(args);
-    k = LENGTH(sa);
-    l = LENGTH(so);
-
     if (!isString(x))
 	error(_("extracting substrings from a non-character object"));
-    len = XLENGTH(x);
+    R_xlen_t len = XLENGTH(x);
     PROTECT(s = allocVector(STRSXP, len));
     if (len > 0) {
+	SEXP sa = CADR(args),
+	    so = CADDR(args);
+	int
+	    k = LENGTH(sa),
+	    l = LENGTH(so);
 	if (!isInteger(sa) || !isInteger(so) || k == 0 || l == 0)
 	    error(_("invalid substring arguments"));
 
-	for (i = 0; i < len; i++) {
-	    start = INTEGER(sa)[i % k];
-	    stop = INTEGER(so)[i % l];
-	    el = STRING_ELT(x,i);
+	for (R_xlen_t i = 0; i < len; i++) {
+	    int start = INTEGER(sa)[i % k],
+		stop  = INTEGER(so)[i % l];
+	    SEXP el = STRING_ELT(x,i);
 	    if (el == NA_STRING || start == NA_INTEGER || stop == NA_INTEGER) {
 		SET_STRING_ELT(s, i, NA_STRING);
 		continue;
 	    }
-	    ienc = getCharCE(el);
-	    ss = CHAR(el);
-	    slen = strlen(ss); /* FIXME -- should handle embedded nuls */
-	    buf = R_AllocStringBuffer(slen+1, &cbuff);
+	    cetype_t ienc = getCharCE(el);
+	    const char *ss = CHAR(el);
+	    size_t slen = strlen(ss); /* FIXME -- should handle embedded nuls */
+	    char *buf = R_AllocStringBuffer(slen+1, &cbuff);
 	    if (start < 1) start = 1;
 	    if (start > stop || start > slen) {
 		buf[0] = '\0';
@@ -348,12 +341,118 @@ SEXP attribute_hidden do_substr(SEXP call, SEXP op, SEXP args, SEXP env)
 	}
 	R_FreeStringBufferL(&cbuff);
     }
-    DUPLICATE_ATTRIB(s, x);
+    SHALLOW_DUPLICATE_ATTRIB(s, x);
     /* This copied the class, if any */
     UNPROTECT(1);
     return s;
 }
 
+// .Internal( startsWith(x, prefix) )  and
+// .Internal( endsWith  (x, suffix) )
+SEXP attribute_hidden
+do_startsWith(SEXP call, SEXP op, SEXP args, SEXP env)
+{
+    checkArity(op, args);
+
+    SEXP x = CAR(args), Xfix = CADR(args); // 'prefix' or 'suffix'
+    if (!isString(x) || !isString(Xfix))
+	error(_("non-character object(s)"));
+    R_xlen_t
+	n1 = XLENGTH(x),
+	n2 = XLENGTH(Xfix),
+	n = (n1 > 0 && n2 > 0) ? ((n1 >= n2) ? n1 : n2) : 0;
+    if (n == 0) return allocVector(LGLSXP, 0);
+    SEXP ans = PROTECT(allocVector(LGLSXP, n));
+
+    typedef const char * cp;
+    if (n2 == 1) { // optimize the most common case
+	SEXP el = STRING_ELT(Xfix, 0);
+	if (el == NA_STRING) {
+	    for (R_xlen_t i = 0; i < n1; i++)
+		LOGICAL(ans)[i] = NA_LOGICAL;
+	} else {
+	    // ASCII matching will do for ASCII Xfix except in non-UTF-8 MBCS
+	    Rboolean need_translate = TRUE;
+	    if (strIsASCII(CHAR(el)) && (utf8locale || !mbcslocale)) 
+		need_translate = FALSE;
+	    cp y0 = need_translate ? translateCharUTF8(el) : CHAR(el);
+	    int ylen = (int) strlen(y0);
+	    for (R_xlen_t i = 0; i < n1; i++) {
+		SEXP el = STRING_ELT(x, i);
+		if (el == NA_STRING) {
+		    LOGICAL(ans)[i] = NA_LOGICAL;
+		} else {
+		    cp x0 = need_translate ? translateCharUTF8(el) : CHAR(el);
+		    if(PRIMVAL(op) == 0) { // startsWith
+			LOGICAL(ans)[i] = strncmp(x0, y0, ylen) == 0;
+		    } else { // endsWith
+			int off = (int)strlen(x0) - ylen;
+			if (off < 0)
+			    LOGICAL(ans)[i] = 0;
+			else {
+			    LOGICAL(ans)[i] = memcmp(x0 + off, y0, ylen) == 0;
+			}
+		    }
+		}
+	    }
+	}
+    } else { // n2 > 1
+	// convert both inputs to UTF-8
+	cp *x0 = (cp *) R_alloc(n1, sizeof(char *));
+	cp *y0 = (cp *) R_alloc(n2, sizeof(char *));
+	// and record lengths, -1 for NA
+	int *x1 = (int *) R_alloc(n1, sizeof(int *));
+	int *y1 = (int *) R_alloc(n2, sizeof(int *));
+	for (R_xlen_t i = 0; i < n1; i++) {
+	    SEXP el = STRING_ELT(x, i);
+	    if (el == NA_STRING)
+		x1[i] = -1;
+	    else {
+		x0[i] = translateCharUTF8(el);
+		x1[i] = (int) strlen(x0[i]);
+	    }
+	}
+	for (R_xlen_t i = 0; i < n2; i++) {
+	    SEXP el = STRING_ELT(Xfix, i);
+	    if (el == NA_STRING)
+		y1[i] = -1;
+	    else {
+		y0[i] = translateCharUTF8(el);
+		y1[i] = (int) strlen(y0[i]);
+	    }
+	}
+	R_xlen_t i, i1, i2;
+	if(PRIMVAL(op) == 0) { // 0 = startsWith, 1 = endsWith
+	    MOD_ITERATE2(n, n1, n2, i, i1, i2, {
+		    if (x1[i1] < 0 || y1[i2] < 0)
+			LOGICAL(ans)[i] = NA_LOGICAL;
+		    else if (x1[i1] < y1[i2])
+			LOGICAL(ans)[i] = 0;
+		    else // memcmp should be faster than strncmp
+			LOGICAL(ans)[i] = 
+			    memcmp(x0[i1], y0[i2], y1[i2]) == 0;
+		});
+	} else { // endsWith
+	    MOD_ITERATE2(n, n1, n2, i, i1, i2, {
+		    if (x1[i1] < 0 || y1[i2] < 0)
+			LOGICAL(ans)[i] = NA_LOGICAL;
+		    else {
+			int off = x1[i1] - y1[i2];
+			if (off < 0)
+			    LOGICAL(ans)[i] = 0;
+			else {
+			    LOGICAL(ans)[i] = 
+				memcmp(x0[i1] + off, y0[i2], y1[i2]) == 0;
+			}
+		    }
+		});
+	}
+    }
+    UNPROTECT(1);
+    return ans;
+}
+
+
 static void
 substrset(char *buf, const char *const str, cetype_t ienc, int sa, int so)
 {
@@ -486,11 +585,11 @@ SEXP attribute_hidden do_substrgets(SEXP call, SEXP op, SEXP args, SEXP env)
 */
 
 
-#define FIRSTCHAR(i) (isspace((int)buff1[i-1]))
-#define LASTCHAR(i) (!isspace((int)buff1[i-1]) && (!buff1[i+1] || isspace((int)buff1[i+1])))
-#define LOWVOW(i) (buff1[i] == 'a' || buff1[i] == 'e' || buff1[i] == 'i' || \
-		   buff1[i] == 'o' || buff1[i] == 'u')
-
+#define FIRSTCHAR(i) (isspace((int)s[i-1]))
+#define LASTCHAR(i) (!isspace((int)s[i-1]) && (!s[i+1] || isspace((int)s[i+1])))
+#define LC_VOWEL(i) (s[i] == 'a' || s[i] == 'e' || s[i] == 'i' || \
+		   s[i] == 'o' || s[i] == 'u')
+#define UPPER (int)(strlen(s) - 1)
 
 /* memmove does allow overlapping src and dest */
 static void mystrcpy(char *dest, const char *src)
@@ -498,132 +597,228 @@ static void mystrcpy(char *dest, const char *src)
     memmove(dest, src, strlen(src)+1);
 }
 
-static SEXP stripchars(const char * const inchar, int minlen)
+static SEXP stripchars(const char * const inchar, int minlen, int usecl)
 {
-/* This routine used to use strcpy with overlapping dest and src.
-   That is not allowed by ISO C.
- */
-    int i, j, nspace = 0, upper;
-    char *buff1 = cbuff.data;
+    int i, j, nspace = 0;
+    char *s = cbuff.data;
 
-    mystrcpy(buff1, inchar);
-    upper = (int)(strlen(buff1) - 1);
+    /* The R wrapper removed leading and trailing spces */
+    mystrcpy(s, inchar);
+    if (strlen(s) < minlen) goto donesc;
 
-    /* remove leading blanks */
-    j = 0;
-    for (i = 0 ; i < upper ; i++)
-	if (isspace((int)buff1[i]))
-	    j++;
-	else
-	    break;
+    /* The for() loops never touch the first character */
 
-    mystrcpy(buff1, &buff1[j]);
-    upper = (int)(strlen(buff1) - 1);
+    /*  record spaces for removal later (as they act as word boundaries) */
+    for (i = UPPER, j = 1; i > 0; i--) {
+	if (isspace((int)s[i])) {
+	    if (j) s[i] = '\0'; // trailing space
+	    else nspace++;
+	} else j = 0;
+	if (strlen(s) - nspace <= minlen)
+	    goto donesc;
+    }
 
-    if (strlen(buff1) < minlen)
-	goto donesc;
-
-    for (i = upper, j = 1; i > 0; i--) {
-	if (isspace((int)buff1[i])) {
-	    if (j)
-		buff1[i] = '\0' ;
-	    else
-		nspace++;
+    if(usecl) {
+	/* remove l/case vowels,
+	   which are not at the beginning of a word but are at the end */
+	for (i = UPPER; i > 0; i--) {
+	    if (LC_VOWEL(i) && LASTCHAR(i))
+		mystrcpy(s + i, s + i + 1);
+	    if (strlen(s) - nspace <= minlen)
+		goto donesc;
 	}
-	else
-	    j = 0;
-	/*strcpy(buff1[i],buff1[i+1]);*/
-	if (strlen(buff1) - nspace <= minlen)
-	    goto donesc;
-    }
 
-    upper = (int)(strlen(buff1) - 1);
-    for (i = upper; i > 0; i--) {
-	if (LOWVOW(i) && LASTCHAR(i))
-	    mystrcpy(&buff1[i], &buff1[i + 1]);
-	if (strlen(buff1) - nspace <= minlen)
-	    goto donesc;
-    }
+	/* remove those not at the beginning of a word */
+	for (i = UPPER; i > 0; i--) {
+	    if (LC_VOWEL(i) && !FIRSTCHAR(i))
+		mystrcpy(s + i, s + i + 1);
+	    if (strlen(s) - nspace <= minlen)
+		goto donesc;
+	}
 
-    upper = (int)(strlen(buff1) - 1);
-    for (i = upper; i > 0; i--) {
-	if (LOWVOW(i) && !FIRSTCHAR(i))
-	    mystrcpy(&buff1[i], &buff1[i + 1]);
-	if (strlen(buff1) - nspace <= minlen)
-	    goto donesc;
-    }
+	/* Now do the same for remaining l/case chars */
+	for (i = UPPER; i > 0; i--) {
+	    if (islower((int)s[i]) && LASTCHAR(i))
+		mystrcpy(s + i, s + i + 1);
+	    if (strlen(s) - nspace <= minlen)
+		goto donesc;
+	}
 
-    upper = (int)(strlen(buff1) - 1);
-    for (i = upper; i > 0; i--) {
-	if (islower((int)buff1[i]) && LASTCHAR(i))
-	    mystrcpy(&buff1[i], &buff1[i + 1]);
-	if (strlen(buff1) - nspace <= minlen)
-	    goto donesc;
-    }
-
-    upper = (int)(strlen(buff1) - 1);
-    for (i = upper; i > 0; i--) {
-	if (islower((int)buff1[i]) && !FIRSTCHAR(i))
-	    mystrcpy(&buff1[i], &buff1[i + 1]);
-	if (strlen(buff1) - nspace <= minlen)
-	    goto donesc;
+	for (i = UPPER; i > 0; i--) {
+	    if (islower((int)s[i]) && !FIRSTCHAR(i))
+		mystrcpy(s + i, s + i + 1);
+	    if (strlen(s) - nspace <= minlen)
+		goto donesc;
+	}
     }
 
     /* all else has failed so we use brute force */
 
-    upper = (int)(strlen(buff1) - 1);
-    for (i = upper; i > 0; i--) {
-	if (!FIRSTCHAR(i) && !isspace((int)buff1[i]))
-	    mystrcpy(&buff1[i], &buff1[i + 1]);
-	if (strlen(buff1) - nspace <= minlen)
+    for (i = UPPER; i > 0; i--) {
+	if (!FIRSTCHAR(i) && !isspace((int)s[i]))
+	    mystrcpy(s + i, s + i + 1);
+	if (strlen(s) - nspace <= minlen)
 	    goto donesc;
     }
 
 donesc:
+    {  // remove internal spaces as required
+	int upper = (int) strlen(s);
+	if (upper > minlen)
+	    for (i = upper - 1; i > 0; i--)
+		if (isspace((int)s[i]))
+		    mystrcpy(s + i, s + i + 1);
+    }
 
-    upper = (int) strlen(buff1);
-    if (upper > minlen)
-	for (i = upper - 1; i > 0; i--)
-	    if (isspace((int)buff1[i]))
-		mystrcpy(&buff1[i], &buff1[i + 1]);
+    return mkChar(s);
+}
 
-    return(mkChar(buff1));
+#define FIRSTCHARW(i) (iswspace((int)wc[i-1]))
+#define LASTCHARW(i) (!iswspace((int)wc[i-1]) && (!wc[i+1] || iswspace((int)wc[i+1])))
+#define WUP (int)(wcslen(wc) - 1)
+
+// lower-case vowels in English plus accented versions
+static int vowels[] = {
+    0x61, 0x65, 0x69, 0x6f, 0x75,
+    0xe0, 0xe1, 0x2e, 0xe3, 0xe4, 0xe5,
+    0xe8, 0xe9, 0xea, 0xeb, 0xec, 0xed, 0xee, 0xef,
+    0xf2, 0xf3, 0xf4, 0xf5, 0xf6, 0xf8, 0xf9, 0xfa, 0xfb, 0xfc,
+    0x101, 0x103, 0x105, 0x113, 0x115, 0x117, 0x118, 0x11b,
+    0x129, 0x12b, 0x12d, 0x12f, 0x131, 0x14d, 0x14f, 0x151,
+    0x169, 0x16b, 0x16d, 0x16f, 0x171, 0x173
+};
+
+static Rboolean iswvowel(wchar_t w)
+{
+    int v = (int) w, n = sizeof(vowels)/sizeof(int);
+    Rboolean found = FALSE;
+    for(int i = 0; i < n; i++)
+	if(v == vowels[i]) {found = TRUE; break;}
+
+    return found;
+}
+
+static void mywcscpy(wchar_t *dest, const wchar_t *src)
+{
+    memmove(dest, src, sizeof(wchar_t) * (wcslen(src)+1));
+}
+
+static SEXP wstripchars(const wchar_t * const inchar, int minlen, int usecl)
+{
+    int i, j, nspace = 0;
+    wchar_t *wc = (wchar_t *)cbuff.data;
+
+    mywcscpy(wc, inchar);
+    if (wcslen(wc) < minlen) goto donewsc;
+
+    for (i = WUP, j = 1; i > 0; i--) {
+	if (iswspace((int)wc[i])) {
+	    if (j) wc[i] = '\0' ; else nspace++;
+	} else j = 0;
+	if (wcslen(wc) - nspace <= minlen)
+	    goto donewsc;
+    }
+
+    if(usecl) {
+	for (i = WUP; i > 0; i--) {
+	    if (iswvowel(wc[i]) && LASTCHARW(i))
+		mywcscpy(wc + i, wc + i + 1);
+	    if (wcslen(wc) - nspace <= minlen)
+		goto donewsc;
+	}
+
+	for (i = WUP; i > 0; i--) {
+	    if (iswvowel(wc[i]) && !FIRSTCHARW(i))
+		mywcscpy(wc + i, wc + i + 1);
+	    if (wcslen(wc) - nspace <= minlen)
+		goto donewsc;
+	}
+
+	for (i = WUP; i > 0; i--) {
+	    if (islower((int)wc[i]) && LASTCHARW(i))
+		mywcscpy(wc + i, wc + i + 1);
+	    if (wcslen(wc) - nspace <= minlen)
+		goto donewsc;
+	}
+
+	for (i = WUP; i > 0; i--) {
+	    if (islower((int)wc[i]) && !FIRSTCHARW(i))
+		mywcscpy(wc + i, wc + i + 1);
+	    if (wcslen(wc) - nspace <= minlen)
+		goto donewsc;
+	}
+    }
+
+    for (i = WUP; i > 0; i--) {
+	if (!FIRSTCHARW(i) && !iswspace((int)wc[i]))
+	    mywcscpy(wc + i, wc + i + 1);
+	if (wcslen(wc) - nspace <= minlen)
+	    goto donewsc;
+    }
+
+donewsc:
+
+    {
+	int upper = (int) wcslen(wc);
+	if (upper > minlen)
+	    for (i = upper - 1; i > 0; i--)
+		if (iswspace((int)wc[i])) mywcscpy(wc + i, wc + i + 1);
+    }
+
+    int nb = (int) wcstoutf8(NULL, wc, 0);
+    char *cbuf = CallocCharBuf(nb+1);
+    wcstoutf8(cbuf, wc, nb + 1);
+    SEXP ans = mkCharCE(cbuf, CE_UTF8);
+    Free(cbuf);
+    return ans;
 }
 
 
 SEXP attribute_hidden do_abbrev(SEXP call, SEXP op, SEXP args, SEXP env)
 {
-    SEXP x, ans;
-    R_xlen_t i, len;
-    Rboolean warn = FALSE;
-    const char *s;
-
     checkArity(op,args);
-    x = CAR(args);
+    SEXP x = CAR(args);
+
     if (!isString(x))
 	error(_("the first argument must be a character vector"));
     int minlen = asInteger(CADR(args));
     if (minlen == NA_INTEGER)
 	error(_("invalid '%s' argument"), "minlength");
+    int usecl = asLogical(CADDR(args));
+    if (usecl == NA_INTEGER)
+	error(_("invalid '%s' argument"), "use.classes");
 
-    len = XLENGTH(x);
-    PROTECT(ans = allocVector(STRSXP, len));
-    const char *vmax = vmaxget();
-    for (i = 0 ; i < len ; i++) {
-	if (STRING_ELT(x, i) == NA_STRING)
+    R_xlen_t len = XLENGTH(x);
+    SEXP ans = PROTECT(allocVector(STRSXP, len));
+    const void *vmax = vmaxget();
+    Rboolean warn = FALSE;
+    for (R_xlen_t i = 0 ; i < len ; i++) {
+	SEXP el = STRING_ELT(x, i);
+	if (el  == NA_STRING)
 	    SET_STRING_ELT(ans, i, NA_STRING);
 	else {
-	    s = translateChar(STRING_ELT(x, i));
-	    if(strlen(s) > minlen) {
-		warn = warn | !strIsASCII(s);
-		R_AllocStringBuffer(strlen(s), &cbuff);
-		SET_STRING_ELT(ans, i, stripchars(s, minlen));
-	    } else SET_STRING_ELT(ans, i, mkChar(s));
+	    const char *s = CHAR(el);
+	    if (strIsASCII(s)) {
+		if(strlen(s) > minlen) {
+		    R_AllocStringBuffer(strlen(s)+1, &cbuff);
+		    SET_STRING_ELT(ans, i, stripchars(s, minlen, usecl));
+		} else SET_STRING_ELT(ans, i, el);
+	    } else {
+		s = translateCharUTF8(el);
+		int nc = (int) utf8towcs(NULL, s, 0);
+		if (nc > minlen) {
+		    warn = TRUE;
+		    const wchar_t *wc = wtransChar(el);
+		    nc = (int) wcslen(wc);
+		    R_AllocStringBuffer(sizeof(wchar_t)*(nc+1), &cbuff);
+		    SET_STRING_ELT(ans, i, wstripchars(wc, minlen, usecl));
+		} else SET_STRING_ELT(ans, i, el);
+	    }
 	}
-	vmaxset(vmax);
+	vmaxset(vmax); // this throws away the result of wtransChar
     }
-    if (warn) warning(_("abbreviate used with non-ASCII chars"));
-    DUPLICATE_ATTRIB(ans, x);
+    if (usecl && warn) warning(_("abbreviate used with non-ASCII chars"));
+    SHALLOW_DUPLICATE_ATTRIB(ans, x);
     /* This copied the class, if any */
     R_FreeStringBufferL(&cbuff);
     UNPROTECT(1);
@@ -825,7 +1020,7 @@ SEXP attribute_hidden do_tolower(SEXP call, SEXP op, SEXP args, SEXP env)
 	    vmaxset(vmax);
 	}
     }
-    DUPLICATE_ATTRIB(y, x);
+    SHALLOW_DUPLICATE_ATTRIB(y, x);
     /* This copied the class, if any */
     UNPROTECT(1);
     return(y);
@@ -1088,13 +1283,13 @@ SEXP attribute_hidden do_chartr(SEXP call, SEXP op, SEXP args, SEXP env)
     _new = CAR(args); args = CDR(args);
     x = CAR(args);
     n = XLENGTH(x);
-    if (!isString(old) || length(old) < 1 || STRING_ELT(old, 0) == NA_STRING)
+    if (!isString(old) || LENGTH(old) < 1 || STRING_ELT(old, 0) == NA_STRING)
 	error(_("invalid '%s' argument"), "old");
-    if (length(old) > 1)
+    if (LENGTH(old) > 1)
 	warning(_("argument '%s' has length > 1 and only the first element will be used"), "old");
-    if (!isString(_new) || length(_new) < 1 || STRING_ELT(_new, 0) == NA_STRING)
+    if (!isString(_new) || LENGTH(_new) < 1 || STRING_ELT(_new, 0) == NA_STRING)
 	error(_("invalid '%s' argument"), "new");
-    if (length(_new) > 1)
+    if (LENGTH(_new) > 1)
 	warning(_("argument '%s' has length > 1 and only the first element will be used"), "new");
     if (!isString(x)) error("invalid '%s' argument", "x");
 
@@ -1301,7 +1496,7 @@ SEXP attribute_hidden do_chartr(SEXP call, SEXP op, SEXP args, SEXP env)
 	vmaxset(vmax);
     }
 
-    DUPLICATE_ATTRIB(y, x);
+    SHALLOW_DUPLICATE_ATTRIB(y, x);
     /* This copied the class, if any */
     UNPROTECT(1);
     return(y);
@@ -1325,44 +1520,47 @@ SEXP attribute_hidden do_strtrim(SEXP call, SEXP op, SEXP args, SEXP env)
     if (!isString(x = CAR(args)))
 	error(_("strtrim() requires a character vector"));
     len = XLENGTH(x);
-    PROTECT(width = coerceVector(CADR(args), INTSXP));
-    nw = LENGTH(width);
-    if (!nw || (nw < len && len % nw))
-	error(_("invalid '%s' argument"), "width");
-    for (i = 0; i < nw; i++)
-	if (INTEGER(width)[i] == NA_INTEGER ||
-	   INTEGER(width)[i] < 0)
-	    error(_("invalid '%s' argument"), "width");
     PROTECT(s = allocVector(STRSXP, len));
-    vmax = vmaxget();
-    for (i = 0; i < len; i++) {
-	if (STRING_ELT(x, i) == NA_STRING) {
-	    SET_STRING_ELT(s, i, STRING_ELT(x, i));
-	    continue;
+    if(len > 0) {
+	PROTECT(width = coerceVector(CADR(args), INTSXP));
+	nw = LENGTH(width);
+	if (!nw || (nw < len && len % nw))
+	    error(_("invalid '%s' argument"), "width");
+	for (i = 0; i < nw; i++)
+	    if (INTEGER(width)[i] == NA_INTEGER ||
+		INTEGER(width)[i] < 0)
+		error(_("invalid '%s' argument"), "width");
+	vmax = vmaxget();
+	for (i = 0; i < len; i++) {
+	    if (STRING_ELT(x, i) == NA_STRING) {
+		SET_STRING_ELT(s, i, STRING_ELT(x, i));
+		continue;
+	    }
+	    w = INTEGER(width)[i % nw];
+	    This = translateChar(STRING_ELT(x, i));
+	    nc = (int) strlen(This);
+	    buf = R_AllocStringBuffer(nc, &cbuff);
+	    wsum = 0;
+	    mbs_init(&mb_st);
+	    for (p = This, w0 = 0, q = buf; *p ;) {
+		nb =  (int) Mbrtowc(&wc, p, MB_CUR_MAX, &mb_st);
+		w0 = Ri18n_wcwidth(wc);
+		if (w0 < 0) { p += nb; continue; } /* skip non-printable chars */
+		wsum += w0;
+		if (wsum <= w) {
+		    for (k = 0; k < nb; k++) *q++ = *p++;
+		} else break;
+	    }
+	    *q = '\0';
+	    SET_STRING_ELT(s, i, markKnown(buf, STRING_ELT(x, i)));
+	    vmaxset(vmax);
 	}
-	w = INTEGER(width)[i % nw];
-	This = translateChar(STRING_ELT(x, i));
-	nc = (int) strlen(This);
-	buf = R_AllocStringBuffer(nc, &cbuff);
-	wsum = 0;
-	mbs_init(&mb_st);
-	for (p = This, w0 = 0, q = buf; *p ;) {
-	    nb =  (int) Mbrtowc(&wc, p, MB_CUR_MAX, &mb_st);
-	    w0 = Ri18n_wcwidth(wc);
-	    if (w0 < 0) { p += nb; continue; } /* skip non-printable chars */
-	    wsum += w0;
-	    if (wsum <= w) {
-		for (k = 0; k < nb; k++) *q++ = *p++;
-	    } else break;
-	}
-	*q = '\0';
-	SET_STRING_ELT(s, i, markKnown(buf, STRING_ELT(x, i)));
-	vmaxset(vmax);
+	R_FreeStringBufferL(&cbuff);
+	UNPROTECT(1);
     }
-    if (len > 0) R_FreeStringBufferL(&cbuff);
-    DUPLICATE_ATTRIB(s, x);
+    SHALLOW_DUPLICATE_ATTRIB(s, x);
     /* This copied the class, if any */
-    UNPROTECT(2);
+    UNPROTECT(1);
     return s;
 }
 
@@ -1392,7 +1590,7 @@ SEXP attribute_hidden do_strtoi(SEXP call, SEXP op, SEXP args, SEXP env)
     x = CAR(args); args = CDR(args);
     b = CAR(args);
 
-    if(!isInteger(b) || (length(b) < 1))
+    if(!isInteger(b) || (LENGTH(b) < 1))
 	error(_("invalid '%s' argument"), "base");
     base = INTEGER(b)[0];
     if((base != 0) && ((base < 2) || (base > 36)))
@@ -1423,3 +1621,64 @@ SEXP attribute_hidden stringSuffix(SEXP string, int fromIndex) {
     UNPROTECT(1); /* res */
     return res;
 }
+
+SEXP attribute_hidden do_strrep(SEXP call, SEXP op, SEXP args, SEXP env)
+{
+    SEXP d, s, x, n;
+    R_xlen_t is, ix, in, ns, nx, nn;
+    const char *xi;
+    int j, ni, nc;
+    const char *cbuf;
+    char *buf;
+    const void *vmax;
+
+    checkArity(op, args);
+
+    x = CAR(args); args = CDR(args);
+    n = CAR(args);
+
+    nx = XLENGTH(x);
+    nn = XLENGTH(n);
+    if((nx == 0) || (nn == 0))
+	return allocVector(STRSXP, 0);
+
+    ns = (nx > nn) ? nx : nn;
+
+    PROTECT(s = allocVector(STRSXP, ns));
+    vmax = vmaxget();
+    is = ix = in = 0;
+    for(; is < ns; is++) {
+	ni = INTEGER(n)[in];
+	if((STRING_ELT(x, ix) == NA_STRING) || (ni == NA_INTEGER)) {
+	    SET_STRING_ELT(s, is, NA_STRING);
+	} else {
+	    if(ni < 0)
+		error(_("invalid '%s' value"), "times");
+	    xi = CHAR(STRING_ELT(x, ix));
+	    nc = (int) strlen(xi);
+
+	    /* check for feasible result length; use double to protect
+	       against integer overflow */
+	    double len = ((double) nc) * ni;
+	    if (len > INT_MAX)
+		error("R character strings are limited to 2^31-1 bytes");
+
+	    cbuf = buf = CallocCharBuf(nc * ni);
+	    for(j = 0; j < ni; j++) {
+		strcpy(buf, xi);
+		buf += nc;
+	    }
+	    SET_STRING_ELT(s, is, markKnown(cbuf, STRING_ELT(x, ix)));
+	    Free(cbuf);
+	    vmaxset(vmax);
+	}
+	ix = (++ix == nx) ? 0 : ix;
+	in = (++in == nn) ? 0 : in;
+    }
+    /* Copy names if not recycled. */
+    if((ns == nx) &&
+       (d = getAttrib(x, R_NamesSymbol)) != R_NilValue)
+	setAttrib(s, R_NamesSymbol, d);
+    UNPROTECT(1);
+    return s;
+}
