@@ -1,7 +1,7 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
  *  Copyright (C) 1995, 1996  Robert Gentleman and Ross Ihaka
- *  Copyright (C) 1999--2014  The R Core Team
+ *  Copyright (C) 1999--2016  The R Core Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -15,7 +15,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, a copy is available at
- *  http://www.r-project.org/Licenses/
+ *  https://www.R-project.org/Licenses/
  */
 
 
@@ -74,20 +74,14 @@
 #endif
 
 #ifdef Win32
-int trio_vsnprintf(char *buffer, size_t bufferSize, const char *format,
-		   va_list args);
-# define vsnprintf trio_vsnprintf
+#include <trioremap.h>
 #endif
-
-extern int R_OutputCon; /* from connections.c */
-
 #ifndef min
 #define min(a, b) (((a)<(b))?(a):(b))
 #endif
 
 #define BUFSIZE 8192  /* used by Rprintf etc */
 
-/* Only if ierr < 0 or not is currently used */
 attribute_hidden
 R_size_t R_Decode2Long(char *p, int *ierr)
 {
@@ -97,6 +91,7 @@ R_size_t R_Decode2Long(char *p, int *ierr)
     /* else look for letter-code ending : */
     if(R_Verbose)
 	REprintf("R_Decode2Long(): v=%ld\n", v);
+    // NOTE: currently, positive *ierr are not differentiated in the callers:
     if(p[0] == 'G') {
 	if((Giga * (double)v) > R_SIZE_T_MAX) { *ierr = 4; return(v); }
 	return (R_size_t) Giga * v;
@@ -125,9 +120,9 @@ R_size_t R_Decode2Long(char *p, int *ierr)
 const char *EncodeLogical(int x, int w)
 {
     static char buff[NB];
-    if(x == NA_LOGICAL) snprintf(buff, NB, "%*s", w, CHAR(R_print.na_string));
-    else if(x) snprintf(buff, NB, "%*s", w, "TRUE");
-    else snprintf(buff, NB, "%*s", w, "FALSE");
+    if(x == NA_LOGICAL) snprintf(buff, NB, "%*s", min(w, (NB-1)), CHAR(R_print.na_string));
+    else if(x) snprintf(buff, NB, "%*s", min(w, (NB-1)), "TRUE");
+    else snprintf(buff, NB, "%*s", min(w, (NB-1)), "FALSE");
     buff[NB-1] = '\0';
     return buff;
 }
@@ -135,7 +130,7 @@ const char *EncodeLogical(int x, int w)
 const char *EncodeInteger(int x, int w)
 {
     static char buff[NB];
-    if(x == NA_INTEGER) snprintf(buff, NB, "%*s", w, CHAR(R_print.na_string));
+    if(x == NA_INTEGER) snprintf(buff, NB, "%*s", min(w, (NB-1)), CHAR(R_print.na_string));
     else snprintf(buff, NB, "%*d", min(w, (NB-1)), x);
     buff[NB-1] = '\0';
     return buff;
@@ -174,16 +169,23 @@ const char *EncodeEnvironment(SEXP x)
 
 const char *EncodeReal(double x, int w, int d, int e, char cdec)
 {
-    static char buff[NB];
-    char *p, fmt[20];
+    char dec[2];
+    dec[0] = cdec; dec[1] = '\0';
+    return EncodeReal0(x, w, d, e, dec);
+}
+
+const char *EncodeReal0(double x, int w, int d, int e, const char *dec)
+{
+    static char buff[NB], buff2[2*NB];
+    char fmt[20], *out = buff;
 
     /* IEEE allows signed zeros (yuck!) */
     if (x == 0.0) x = 0.0;
     if (!R_FINITE(x)) {
-	if(ISNA(x)) snprintf(buff, NB, "%*s", w, CHAR(R_print.na_string));
-	else if(ISNAN(x)) snprintf(buff, NB, "%*s", w, "NaN");
-	else if(x > 0) snprintf(buff, NB, "%*s", w, "Inf");
-	else snprintf(buff, NB, "%*s", w, "-Inf");
+	if(ISNA(x)) snprintf(buff, NB, "%*s", min(w, (NB-1)), CHAR(R_print.na_string));
+	else if(ISNAN(x)) snprintf(buff, NB, "%*s", min(w, (NB-1)), "NaN");
+	else if(x > 0) snprintf(buff, NB, "%*s", min(w, (NB-1)), "Inf");
+	else snprintf(buff, NB, "%*s", min(w, (NB-1)), "-Inf");
     }
     else if (e) {
 	if(d) {
@@ -201,12 +203,85 @@ const char *EncodeReal(double x, int w, int d, int e, char cdec)
     }
     buff[NB-1] = '\0';
 
-    if(cdec != '.')
-      for(p = buff; *p; p++) if(*p == '.') *p = cdec;
+    if(strcmp(dec, ".")) {
+	char *p, *q;
+	for(p = buff, q = buff2; *p; p++) {
+	    if(*p == '.') for(const char *r = dec; *r; r++) *q++ = *r;
+	    else *q++ = *p;
+	}
+	*q = '\0';
+	out = buff2;
+    }
 
-    return buff;
+    return out;
 }
 
+static const char
+*EncodeRealDrop0(double x, int w, int d, int e, const char *dec)
+{
+    static char buff[NB], buff2[2*NB];
+    char fmt[20], *out = buff;
+
+    /* IEEE allows signed zeros (yuck!) */
+    if (x == 0.0) x = 0.0;
+    if (!R_FINITE(x)) {
+	if(ISNA(x)) snprintf(buff, NB, "%*s", min(w, (NB-1)), CHAR(R_print.na_string));
+	else if(ISNAN(x)) snprintf(buff, NB, "%*s", min(w, (NB-1)), "NaN");
+	else if(x > 0) snprintf(buff, NB, "%*s", min(w, (NB-1)), "Inf");
+	else snprintf(buff, NB, "%*s", min(w, (NB-1)), "-Inf");
+    }
+    else if (e) {
+	if(d) {
+	    sprintf(fmt,"%%#%d.%de", min(w, (NB-1)), d);
+	    snprintf(buff, NB, fmt, x);
+	}
+	else {
+	    sprintf(fmt,"%%%d.%de", min(w, (NB-1)), d);
+	    snprintf(buff, NB, fmt, x);
+	}
+    }
+    else { /* e = 0 */
+	sprintf(fmt,"%%%d.%df", min(w, (NB-1)), d);
+	snprintf(buff, NB, fmt, x);
+    }
+    buff[NB-1] = '\0';
+
+    // Drop trailing zeroes
+    for (char *p = buff; *p; p++) {
+	if(*p == '.') {
+	    char *replace = p++;
+	    while ('0' <= *p  &&  *p <= '9')
+		if(*(p++) != '0')
+		    replace = p;
+	    if(replace != p)
+		while((*(replace++) = *(p++)))
+		    ;
+	    break;
+	}
+    }
+
+    if(strcmp(dec, ".")) {
+	char *p, *q;
+	for(p = buff, q = buff2; *p; p++) {
+	    if(*p == '.') for(const char *r = dec; *r; r++) *q++ = *r;
+	    else *q++ = *p;
+	}
+	*q = '\0';
+	out = buff2;
+    }
+
+    return out;
+}
+
+SEXP attribute_hidden StringFromReal(double x, int *warn)
+{
+    int w, d, e;
+    formatReal(&x, 1, &w, &d, &e, 0);
+    if (ISNA(x)) return NA_STRING;
+    else return mkChar(EncodeRealDrop0(x, w, d, e, OutDec));
+}
+
+
 attribute_hidden
 const char *EncodeReal2(double x, int w, int d, int e)
 {
@@ -216,10 +291,10 @@ const char *EncodeReal2(double x, int w, int d, int e)
     /* IEEE allows signed zeros (yuck!) */
     if (x == 0.0) x = 0.0;
     if (!R_FINITE(x)) {
-	if(ISNA(x)) snprintf(buff, NB, "%*s", w, CHAR(R_print.na_string));
-	else if(ISNAN(x)) snprintf(buff, NB, "%*s", w, "NaN");
-	else if(x > 0) snprintf(buff, NB, "%*s", w, "Inf");
-	else snprintf(buff, NB, "%*s", w, "-Inf");
+	if(ISNA(x)) snprintf(buff, NB, "%*s", min(w, (NB-1)), CHAR(R_print.na_string));
+	else if(ISNAN(x)) snprintf(buff, NB, "%*s", min(w, (NB-1)), "NaN");
+	else if(x > 0) snprintf(buff, NB, "%*s", min(w, (NB-1)), "Inf");
+	else snprintf(buff, NB, "%*s", min(w, (NB-1)), "-Inf");
     }
     else if (e) {
 	if(d) {
@@ -243,13 +318,9 @@ void z_prec_r(Rcomplex *r, Rcomplex *x, double digits);
 
 const char
 *EncodeComplex(Rcomplex x, int wr, int dr, int er, int wi, int di, int ei,
-	       char cdec)
+	       const char *dec)
 {
     static char buff[NB];
-    char Re[NB];
-    const char *Im, *tmp;
-    int flagNegIm = 0;
-    Rcomplex y;
 
     /* IEEE allows signed zeros; strip these here */
     if (x.r == 0.0) x.r = 0.0;
@@ -258,8 +329,12 @@ const char
     if (ISNA(x.r) || ISNA(x.i)) {
 	snprintf(buff, NB,
 		 "%*s", /* was "%*s%*s", R_print.gap, "", */
-		 wr+wi+2, CHAR(R_print.na_string));
+		 min(wr+wi+2, (NB-1)), CHAR(R_print.na_string));
     } else {
+	char Re[NB];
+	const char *Im, *tmp;
+	int flagNegIm = 0;
+	Rcomplex y;
 	/* formatComplex rounded, but this does not, and we need to
 	   keep it that way so we don't get strange trailing zeros.
 	   But we do want to avoid printing small exponentials that
@@ -267,12 +342,10 @@ const char
 	 */
 	z_prec_r(&y, &x, R_print.digits);
 	/* EncodeReal has static buffer, so copy */
-	if(y.r == 0.) tmp = EncodeReal(y.r, wr, dr, er, cdec);
-	else tmp = EncodeReal(x.r, wr, dr, er, cdec);
+	tmp = EncodeReal0(y.r == 0. ? y.r : x.r, wr, dr, er, dec);
 	strcpy(Re, tmp);
 	if ( (flagNegIm = (x.i < 0)) ) x.i = -x.i;
-	if(y.i == 0.) Im = EncodeReal(y.i, wi, di, ei, cdec);
-	else Im = EncodeReal(x.i, wi, di, ei, cdec);
+	Im = EncodeReal0(y.i == 0. ? y.i : x.i, wi, di, ei, dec);
 	snprintf(buff, NB, "%s%s%si", Re, flagNegIm ? "-" : "+", Im);
     }
     buff[NB-1] = '\0';
@@ -288,8 +361,7 @@ const char
 
    On Windows with surrogate pairs it will not be canonical, but AFAIK
    they do not occur in any MBCS (so it would only matter if we implement
-   UTF-8, and then only if Windows has surrogate pairs switched on,
-   which Western versions at least do not.).
+   UTF-8, and then only if Windows has surrogate pairs switched on.).
 */
 
 #include <rlocale.h> /* redefines isw* functions */
@@ -298,10 +370,11 @@ const char
 #include "rgui_UTF8.h"
 #endif
 
-/* strlen() using escaped rather than literal form,
-   and allowing for embedded nuls.
+/* strlen() using escaped rather than literal form.
    In MBCS locales it works in characters, and reports in display width.
-   Also used in printarray.c.
+   Rstrwid is also used in printarray.c.
+
+   This supported embedded nuls when we had those.
  */
 attribute_hidden
 int Rstrwid(const char *str, int slen, cetype_t ienc, int quote)
@@ -309,6 +382,18 @@ int Rstrwid(const char *str, int slen, cetype_t ienc, int quote)
     const char *p = str;
     int len = 0, i;
 
+    if(ienc == CE_BYTES) { // not currently used for that encoding
+	for (i = 0; i < slen; i++) {
+	    unsigned char k = str[i];
+	    if (k >= 0x20 && k < 0x80) len += 1;
+	    else len += 4;
+	}
+	return len;
+    }
+    /* Future-proof: currently that is all Rstrlen calls it with,
+       and printarray has CE_NATIVE explicitly */
+    if(ienc > 2) // CE_NATIVE, CE_UTF8, CE_BYTES are supported
+	warning("unsupported encoding (%d) in Rstrwid", ienc);
     if(mbcslocale || ienc == CE_UTF8) {
 	int res;
 	mbstate_t mb_st;
@@ -367,10 +452,10 @@ int Rstrwid(const char *str, int slen, cetype_t ienc, int quote)
 		p++;
 	    }
 	}
-    } else
+    } else // not MBCS nor marked as UTF-8
 	for (i = 0; i < slen; i++) {
-	    /* ASCII */
 	    if((unsigned char) *p < 0x80) {
+		/* ASCII */
 		if(isprint((int)*p)) {
 		    switch(*p) {
 		    case '\\':
@@ -410,17 +495,25 @@ int Rstrwid(const char *str, int slen, cetype_t ienc, int quote)
     return len;
 }
 
+/* Match what EncodeString does with encodings */
 attribute_hidden
 int Rstrlen(SEXP s, int quote)
 {
-    return Rstrwid(CHAR(s), LENGTH(s), getCharCE(s), quote);
+    cetype_t ienc = getCharCE(s);
+    if (ienc == CE_UTF8 || ienc == CE_BYTES)
+	return Rstrwid(CHAR(s), LENGTH(s), ienc, quote);
+    const void *vmax = vmaxget();
+    const char *p = translateChar(s);
+    int len = Rstrwid(p, (int)strlen(p), CE_NATIVE, quote);
+    vmaxset(vmax);
+    return len;
 }
 
 /* Here w is the minimum field width
    If 'quote' is non-zero the result should be quoted (and internal quotes
    escaped and NA strings handled differently).
 
-   EncodeString is called from EncodeElements, cat() (for labels when
+   EncodeString is called from EncodeElement, cat() (for labels when
    filling), to (auto)print character vectors, arrays, names and
    CHARSXPs.  It is also called by do_encodeString, but not from
    format().
@@ -548,13 +641,15 @@ const char *EncodeString(SEXP s, int w, int quote, Rprt_adj justify)
 		if(res == 0) {k = 0; wc = L'\0';}
 		if(0x20 <= k && k < 0x7f && iswprint(wc)) {
 		    switch(wc) {
-		    case L'\\': *q++ = '\\'; *q++ = '\\'; p++;
-			break;
+		    case L'\\': *q++ = '\\'; *q++ = '\\'; p++; break;
 		    case L'\'':
 		    case L'"':
 		    case L'`':
-			if(quote == *p)  *q++ = '\\'; *q++ = *p++;
-			break;
+			{
+			    if(quote == *p)  *q++ = '\\';
+			    *q++ = *p++;
+			    break;
+			}
 		    default:
 			for(j = 0; j < res; j++) *q++ = *p++;
 			break;
@@ -573,7 +668,8 @@ const char *EncodeString(SEXP s, int w, int quote, Rprt_adj justify)
 
 		    default:
 			/* print in octal */
-			snprintf(buf, 5, "\\%03o", k);
+			// gcc 7 requires cast here
+			snprintf(buf, 5, "\\%03o", (unsigned char)k);
 			for(j = 0; j < 4; j++) *q++ = buf[j];
 			break;
 		    }
@@ -624,7 +720,11 @@ const char *EncodeString(SEXP s, int w, int quote, Rprt_adj justify)
 		    case '\'':
 		    case '"':
 		    case '`':
-			if(quote == *p)  *q++ = '\\'; *q++ = *p; break;
+		    {
+			if(quote == *p)  *q++ = '\\';
+			*q++ = *p;
+			break;
+		    }
 		    default: *q++ = *p; break;
 		    }
 		} else switch(*p) {
@@ -674,9 +774,16 @@ const char *EncodeString(SEXP s, int w, int quote, Rprt_adj justify)
 
 /* EncodeElement is called by cat(), write.table() and deparsing. */
 
-/* NB this is called by R.app even though it is in no public header, so 
+/* NB this is called by R.app even though it is in no public header, so
    alter there if you alter this */
-const char *EncodeElement(SEXP x, int indx, int quote, char dec)
+const char *EncodeElement(SEXP x, int indx, int quote, char cdec)
+{
+    char dec[2];
+    dec[0] = cdec; dec[1] = '\0';
+    return EncodeElement0(x, indx, quote, dec);
+}
+
+const char *EncodeElement0(SEXP x, int indx, int quote, const char *dec)
 {
     int w, d, e, wi, di, ei;
     const char *res;
@@ -692,7 +799,7 @@ const char *EncodeElement(SEXP x, int indx, int quote, char dec)
 	break;
     case REALSXP:
 	formatReal(&REAL(x)[indx], 1, &w, &d, &e, 0);
-	res = EncodeReal(REAL(x)[indx], w, d, e, dec);
+	res = EncodeReal0(REAL(x)[indx], w, d, e, dec);
 	break;
     case STRSXP:
 	formatString(&STRING_PTR(x)[indx], 1, &w, quote);
