@@ -1,7 +1,7 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
  *  Copyright (C) 1995, 1996  Robert Gentleman and Ross Ihaka
- *  Copyright (C) 1997--2012  The R Core Team
+ *  Copyright (C) 1997--2015  The R Core Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -15,7 +15,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, a copy is available at
- *  http://www.r-project.org/Licenses/
+ *  https://www.R-project.org/Licenses/
  */
 
 
@@ -27,6 +27,7 @@
 #include <Internal.h>
 #include <Rmath.h>
 #include <errno.h>
+#include <R_ext/Itermacros.h>
 
 /* interval at which to check interrupts, a guess */
 #define NINTERRUPT 10000000
@@ -37,14 +38,73 @@ static SEXP complex_relop(RELOP_TYPE code, SEXP s1, SEXP s2, SEXP call);
 static SEXP string_relop(RELOP_TYPE code, SEXP s1, SEXP s2);
 static SEXP raw_relop(RELOP_TYPE code, SEXP s1, SEXP s2);
 
+#define DO_SCALAR_RELOP(oper, x, y) do {			\
+	switch (oper) {						\
+	case EQOP: return ScalarLogical((x) == (y));		\
+	case NEOP: return ScalarLogical((x) != (y));		\
+	case LTOP: return ScalarLogical((x) < (y));		\
+	case GTOP: return ScalarLogical((x) > (y));		\
+	case LEOP: return ScalarLogical((x) <= (y));		\
+	case GEOP: return ScalarLogical((x) >= (y));		\
+	}							\
+    } while (0)
+
 SEXP attribute_hidden do_relop(SEXP call, SEXP op, SEXP args, SEXP env)
 {
-    SEXP ans;
+    SEXP ans, arg1, arg2;
+    int argc;
+    int oper = PRIMVAL(op);
 
-    if (DispatchGroup("Ops", call, op, args, env, &ans))
-	return ans;
-    checkArity(op, args);
-    return do_relop_dflt(call, op, CAR(args), CADR(args));
+    if (args != R_NilValue &&
+	CDR(args) != R_NilValue &&
+	CDDR(args) == R_NilValue)
+	argc = 2;
+    else
+	argc = length(args);
+    arg1 = CAR(args);
+    arg2 = CADR(args);
+
+    if (ATTRIB(arg1) != R_NilValue || ATTRIB(arg2) != R_NilValue) {
+	if (DispatchGroup("Ops", call, op, args, env, &ans))
+	    return ans;
+	if (argc != 2)
+	    error("operator needs two arguments");
+    }
+    else if (argc == 2) {
+	if (IS_SCALAR(arg1, INTSXP)) {
+	    int ix = INTEGER(arg1)[0];
+	    if (IS_SCALAR(arg2, INTSXP)) {
+		int iy = INTEGER(arg2)[0];
+		if (ix == NA_INTEGER || iy == NA_INTEGER)
+		    return ScalarLogical(NA_LOGICAL);
+		DO_SCALAR_RELOP(oper, ix, iy);
+	    }
+	    else if (IS_SCALAR(arg2, REALSXP)) {
+		double dy = REAL(arg2)[0];
+		if (ix == NA_INTEGER || ISNAN(dy))
+		    return ScalarLogical(NA_LOGICAL);
+		DO_SCALAR_RELOP(oper, ix, dy);
+	    }
+	}
+	else if (IS_SCALAR(arg1, REALSXP)) {
+	    double dx = REAL(arg1)[0];
+	    if (IS_SCALAR(arg2, INTSXP)) {
+		int iy = INTEGER(arg2)[0];
+		if (ISNAN(dx) || iy == NA_INTEGER)
+		    return ScalarLogical(NA_LOGICAL);
+		DO_SCALAR_RELOP(oper, dx, iy);
+	    }
+	    else if (IS_SCALAR(arg2, REALSXP)) {
+		double dy = REAL(arg2)[0];
+		if (ISNAN(dx) || ISNAN(dy))
+		    return ScalarLogical(NA_LOGICAL);
+		DO_SCALAR_RELOP(oper, dx, dy);
+	    }
+	}
+    }
+    else error("operator needs two arguments");
+
+    return do_relop_dflt(call, op, arg1, arg2);
 }
 
 SEXP attribute_hidden do_relop_dflt(SEXP call, SEXP op, SEXP x, SEXP y)
@@ -54,29 +114,69 @@ SEXP attribute_hidden do_relop_dflt(SEXP call, SEXP op, SEXP x, SEXP y)
     int xarray, yarray, xts, yts;
     Rboolean mismatch = FALSE, iS;
     PROTECT_INDEX xpi, ypi;
+    SEXPTYPE typex, typey;
 
-    PROTECT_WITH_INDEX(x, &xpi);
-    PROTECT_WITH_INDEX(y, &ypi);
     nx = xlength(x);
     ny = xlength(y);
+    typex = TYPEOF(x);
+    typey = TYPEOF(y);
 
     /* pre-test to handle the most common case quickly.
        Used to skip warning too ....
      */
     if (ATTRIB(x) == R_NilValue && ATTRIB(y) == R_NilValue &&
-	TYPEOF(x) == REALSXP && TYPEOF(y) == REALSXP && nx > 0 && ny > 0) {
-	SEXP ans = real_relop((RELOP_TYPE) PRIMVAL(op), x, y);
+	(typex == REALSXP || typex == INTSXP) &&
+	(typey == REALSXP || typey == INTSXP) &&
+	nx > 0 && ny > 0) {
+
+	/* handle the scalar case */
+	if (nx == 1 && ny == 1) {
+	    if (typex == INTSXP && typey == INTSXP) {
+		int ix = INTEGER(x)[0];
+		int iy = INTEGER(y)[0];
+		if (ix == NA_INTEGER || iy == NA_INTEGER)
+		    return ScalarLogical(NA_LOGICAL);
+		DO_SCALAR_RELOP(PRIMVAL(op), ix, iy);
+	    }
+	    else {
+		double dx = typex == REALSXP ? REAL(x)[0] :
+		    INTEGER(x)[0] != NA_INTEGER ? INTEGER(x)[0] : NA_REAL;
+		double dy = typey == REALSXP ? REAL(y)[0] :
+		    INTEGER(y)[0] != NA_INTEGER ? INTEGER(y)[0] : NA_REAL;
+		if (ISNAN(dx) || ISNAN(dy))
+		    return ScalarLogical(NA_LOGICAL);
+		DO_SCALAR_RELOP(PRIMVAL(op), dx, dy);
+	    }
+	}
+
+	/* non-scalar case */
+	PROTECT_WITH_INDEX(x, &xpi);
+	PROTECT_WITH_INDEX(y, &ypi);
+	SEXP ans;
+	if (typex == INTSXP && typey == INTSXP)
+	    ans = integer_relop(PRIMVAL(op), x, y);
+	else {
+	    if (typex == INTSXP)
+		REPROTECT(x = coerceVector(x, REALSXP), xpi);
+	    if (typey == INTSXP)
+		REPROTECT(y = coerceVector(y, REALSXP), ypi);
+	    ans = real_relop(PRIMVAL(op), x, y);
+	}
 	if (nx > 0 && ny > 0)
 	    mismatch = ((nx > ny) ? nx % ny : ny % nx) != 0;
 	if (mismatch) {
 	    PROTECT(ans);
-	    warningcall(call, _("longer object length is not a multiple of shorter object length"));
+	    warningcall(call, _("longer object length is not "
+				"a multiple of shorter object length"));
 	    UNPROTECT(1);
 	}
 	UNPROTECT(2);
 	return ans;
     }
 
+    PROTECT_WITH_INDEX(x, &xpi);
+    PROTECT_WITH_INDEX(y, &ypi);
+
     /* That symbols and calls were allowed was undocumented prior to
        R 2.5.0.  We deparse them as deparse() would, minus attributes */
     if ((iS = isSymbol(x)) || TYPEOF(x) == LANGSXP) {
@@ -223,15 +323,6 @@ SEXP attribute_hidden do_relop_dflt(SEXP call, SEXP op, SEXP x, SEXP y)
     return x;
 }
 
-/* i1 = i % n1; i2 = i % n2;
- * this macro is quite a bit faster than having real modulo calls
- * in the loop (tested on Intel and Sparc)
- */
-#define mod_iterate(n1,n2,i1,i2) for (i=i1=i2=0; i<n; \
-	i1 = (++i1 == n1) ? 0 : i1,\
-	i2 = (++i2 == n2) ? 0 : i2,\
-	++i)
-
 static SEXP integer_relop(RELOP_TYPE code, SEXP s1, SEXP s2)
 {
     R_xlen_t i, i1, i2, n, n1, n2;
@@ -247,7 +338,7 @@ static SEXP integer_relop(RELOP_TYPE code, SEXP s1, SEXP s2)
 
     switch (code) {
     case EQOP:
-	mod_iterate(n1, n2, i1, i2) {
+	MOD_ITERATE2(n, n1, n2, i, i1, i2, {
 //	    if ((i+1) % NINTERRUPT == 0) R_CheckUserInterrupt();
 	    x1 = INTEGER(s1)[i1];
 	    x2 = INTEGER(s2)[i2];
@@ -255,10 +346,10 @@ static SEXP integer_relop(RELOP_TYPE code, SEXP s1, SEXP s2)
 		LOGICAL(ans)[i] = NA_LOGICAL;
 	    else
 		LOGICAL(ans)[i] = (x1 == x2);
-	}
+	});
 	break;
     case NEOP:
-	mod_iterate(n1, n2, i1, i2) {
+	MOD_ITERATE2(n, n1, n2, i, i1, i2, {
 //	    if ((i+1) % NINTERRUPT == 0) R_CheckUserInterrupt();
 	    x1 = INTEGER(s1)[i1];
 	    x2 = INTEGER(s2)[i2];
@@ -266,10 +357,10 @@ static SEXP integer_relop(RELOP_TYPE code, SEXP s1, SEXP s2)
 		LOGICAL(ans)[i] = NA_LOGICAL;
 	    else
 		LOGICAL(ans)[i] = (x1 != x2);
-	}
+	});
 	break;
     case LTOP:
-	mod_iterate(n1, n2, i1, i2) {
+	MOD_ITERATE2(n, n1, n2, i, i1, i2, {
 //	    if ((i+1) % NINTERRUPT == 0) R_CheckUserInterrupt();
 	    x1 = INTEGER(s1)[i1];
 	    x2 = INTEGER(s2)[i2];
@@ -277,10 +368,10 @@ static SEXP integer_relop(RELOP_TYPE code, SEXP s1, SEXP s2)
 		LOGICAL(ans)[i] = NA_LOGICAL;
 	    else
 		LOGICAL(ans)[i] = (x1 < x2);
-	}
+	});
 	break;
     case GTOP:
-	mod_iterate(n1, n2, i1, i2) {
+	MOD_ITERATE2(n, n1, n2, i, i1, i2, {
 //	    if ((i+1) % NINTERRUPT == 0) R_CheckUserInterrupt();
 	    x1 = INTEGER(s1)[i1];
 	    x2 = INTEGER(s2)[i2];
@@ -288,10 +379,10 @@ static SEXP integer_relop(RELOP_TYPE code, SEXP s1, SEXP s2)
 		LOGICAL(ans)[i] = NA_LOGICAL;
 	    else
 		LOGICAL(ans)[i] = (x1 > x2);
-	}
+	});
 	break;
     case LEOP:
-	mod_iterate(n1, n2, i1, i2) {
+	MOD_ITERATE2(n, n1, n2, i, i1, i2, {
 //	    if ((i+1) % NINTERRUPT == 0) R_CheckUserInterrupt();
 	    x1 = INTEGER(s1)[i1];
 	    x2 = INTEGER(s2)[i2];
@@ -299,10 +390,10 @@ static SEXP integer_relop(RELOP_TYPE code, SEXP s1, SEXP s2)
 		LOGICAL(ans)[i] = NA_LOGICAL;
 	    else
 		LOGICAL(ans)[i] = (x1 <= x2);
-	}
+	});
 	break;
     case GEOP:
-	mod_iterate(n1, n2, i1, i2) {
+	MOD_ITERATE2(n, n1, n2, i, i1, i2, {
 //	    if ((i+1) % NINTERRUPT == 0) R_CheckUserInterrupt();
 	    x1 = INTEGER(s1)[i1];
 	    x2 = INTEGER(s2)[i2];
@@ -310,7 +401,7 @@ static SEXP integer_relop(RELOP_TYPE code, SEXP s1, SEXP s2)
 		LOGICAL(ans)[i] = NA_LOGICAL;
 	    else
 		LOGICAL(ans)[i] = (x1 >= x2);
-	}
+	});
 	break;
     }
     UNPROTECT(2);
@@ -332,7 +423,7 @@ static SEXP real_relop(RELOP_TYPE code, SEXP s1, SEXP s2)
 
     switch (code) {
     case EQOP:
-	mod_iterate(n1, n2, i1, i2) {
+	MOD_ITERATE2(n, n1, n2, i, i1, i2, {
 //	    if ((i+1) % NINTERRUPT == 0) R_CheckUserInterrupt();
 	    x1 = REAL(s1)[i1];
 	    x2 = REAL(s2)[i2];
@@ -340,10 +431,10 @@ static SEXP real_relop(RELOP_TYPE code, SEXP s1, SEXP s2)
 		LOGICAL(ans)[i] = NA_LOGICAL;
 	    else
 		LOGICAL(ans)[i] = (x1 == x2);
-	}
+	});
 	break;
     case NEOP:
-	mod_iterate(n1, n2, i1, i2) {
+	MOD_ITERATE2(n, n1, n2, i, i1, i2, {
 //	    if ((i+1) % NINTERRUPT == 0) R_CheckUserInterrupt();
 	    x1 = REAL(s1)[i1];
 	    x2 = REAL(s2)[i2];
@@ -351,10 +442,10 @@ static SEXP real_relop(RELOP_TYPE code, SEXP s1, SEXP s2)
 		LOGICAL(ans)[i] = NA_LOGICAL;
 	    else
 		LOGICAL(ans)[i] = (x1 != x2);
-	}
+	});
 	break;
     case LTOP:
-	mod_iterate(n1, n2, i1, i2) {
+	MOD_ITERATE2(n, n1, n2, i, i1, i2, {
 //	    if ((i+1) % NINTERRUPT == 0) R_CheckUserInterrupt();
 	    x1 = REAL(s1)[i1];
 	    x2 = REAL(s2)[i2];
@@ -362,10 +453,10 @@ static SEXP real_relop(RELOP_TYPE code, SEXP s1, SEXP s2)
 		LOGICAL(ans)[i] = NA_LOGICAL;
 	    else
 		LOGICAL(ans)[i] = (x1 < x2);
-	}
+	});
 	break;
     case GTOP:
-	mod_iterate(n1, n2, i1, i2) {
+	MOD_ITERATE2(n, n1, n2, i, i1, i2, {
 //	    if ((i+1) % NINTERRUPT == 0) R_CheckUserInterrupt();
 	    x1 = REAL(s1)[i1];
 	    x2 = REAL(s2)[i2];
@@ -373,10 +464,10 @@ static SEXP real_relop(RELOP_TYPE code, SEXP s1, SEXP s2)
 		LOGICAL(ans)[i] = NA_LOGICAL;
 	    else
 		LOGICAL(ans)[i] = (x1 > x2);
-	}
+	});
 	break;
     case LEOP:
-	mod_iterate(n1, n2, i1, i2) {
+	MOD_ITERATE2(n, n1, n2, i, i1, i2, {
 //	    if ((i+1) % NINTERRUPT == 0) R_CheckUserInterrupt();
 	    x1 = REAL(s1)[i1];
 	    x2 = REAL(s2)[i2];
@@ -384,10 +475,10 @@ static SEXP real_relop(RELOP_TYPE code, SEXP s1, SEXP s2)
 		LOGICAL(ans)[i] = NA_LOGICAL;
 	    else
 		LOGICAL(ans)[i] = (x1 <= x2);
-	}
+	});
 	break;
     case GEOP:
-	mod_iterate(n1, n2, i1, i2) {
+	MOD_ITERATE2(n, n1, n2, i, i1, i2, {
 //	    if ((i+1) % NINTERRUPT == 0) R_CheckUserInterrupt();
 	    x1 = REAL(s1)[i1];
 	    x2 = REAL(s2)[i2];
@@ -395,7 +486,7 @@ static SEXP real_relop(RELOP_TYPE code, SEXP s1, SEXP s2)
 		LOGICAL(ans)[i] = NA_LOGICAL;
 	    else
 		LOGICAL(ans)[i] = (x1 >= x2);
-	}
+	});
 	break;
     }
     UNPROTECT(2);
@@ -421,7 +512,7 @@ static SEXP complex_relop(RELOP_TYPE code, SEXP s1, SEXP s2, SEXP call)
 
     switch (code) {
     case EQOP:
-	mod_iterate(n1, n2, i1, i2) {
+	MOD_ITERATE2(n, n1, n2, i, i1, i2, {
 //	    if ((i+1) % NINTERRUPT == 0) R_CheckUserInterrupt();
 	    x1 = COMPLEX(s1)[i1];
 	    x2 = COMPLEX(s2)[i2];
@@ -430,10 +521,10 @@ static SEXP complex_relop(RELOP_TYPE code, SEXP s1, SEXP s2, SEXP call)
 		LOGICAL(ans)[i] = NA_LOGICAL;
 	    else
 		LOGICAL(ans)[i] = (x1.r == x2.r && x1.i == x2.i);
-	}
+	});
 	break;
     case NEOP:
-	mod_iterate(n1, n2, i1, i2) {
+	MOD_ITERATE2(n, n1, n2, i, i1, i2, {
 //	    if ((i+1) % NINTERRUPT == 0) R_CheckUserInterrupt();
 	    x1 = COMPLEX(s1)[i1];
 	    x2 = COMPLEX(s2)[i2];
@@ -442,7 +533,7 @@ static SEXP complex_relop(RELOP_TYPE code, SEXP s1, SEXP s2, SEXP call)
 		LOGICAL(ans)[i] = NA_LOGICAL;
 	    else
 		LOGICAL(ans)[i] = (x1.r != x2.r || x1.i != x2.i);
-	}
+	});
 	break;
     default:
 	/* never happens (-Wall) */
@@ -457,7 +548,7 @@ static SEXP complex_relop(RELOP_TYPE code, SEXP s1, SEXP s2, SEXP call)
    outside the collation domain. */
 static SEXP string_relop(RELOP_TYPE code, SEXP s1, SEXP s2)
 {
-    R_xlen_t i, n, n1, n2, res;
+    R_xlen_t i, n, n1, n2, res, i1, i2;
     SEXP ans, c1, c2;
     const void *vmax = vmaxget(); // for Scollate
 
@@ -470,32 +561,32 @@ static SEXP string_relop(RELOP_TYPE code, SEXP s1, SEXP s2)
 
     switch (code) {
     case EQOP:
-	for (i = 0; i < n; i++) {
+	MOD_ITERATE2(n, n1, n2, i, i1, i2, {
 //	    if ((i+1) % NINTERRUPT == 0) R_CheckUserInterrupt();
-	    c1 = STRING_ELT(s1, i % n1);
-	    c2 = STRING_ELT(s2, i % n2);
+	    c1 = STRING_ELT(s1, i1);
+	    c2 = STRING_ELT(s2, i2);
 	    if (c1 == NA_STRING || c2 == NA_STRING)
 		LOGICAL(ans)[i] = NA_LOGICAL;
 	    else
 		LOGICAL(ans)[i] = Seql(c1, c2) ? 1 : 0;
-	}
+	});
 	break;
     case NEOP:
-	for (i = 0; i < n; i++) {
+	MOD_ITERATE2(n, n1, n2, i, i1, i2, {
 //	    if ((i+1) % NINTERRUPT == 0) R_CheckUserInterrupt();
-	    c1 = STRING_ELT(s1, i % n1);
-	    c2 = STRING_ELT(s2, i % n2);
+	    c1 = STRING_ELT(s1, i1);
+	    c2 = STRING_ELT(s2, i2);
 	    if (c1 == NA_STRING || c2 == NA_STRING)
 		LOGICAL(ans)[i] = NA_LOGICAL;
 	    else
 		LOGICAL(ans)[i] = Seql(c1, c2) ? 0 : 1;
-	}
+	});
 	break;
     case LTOP:
-	for (i = 0; i < n; i++) {
+	MOD_ITERATE2(n, n1, n2, i, i1, i2, {
 //	    if ((i+1) % NINTERRUPT == 0) R_CheckUserInterrupt();
-	    c1 = STRING_ELT(s1, i % n1);
-	    c2 = STRING_ELT(s2, i % n2);
+	    c1 = STRING_ELT(s1, i1);
+	    c2 = STRING_ELT(s2, i2);
 	    if (c1 == NA_STRING || c2 == NA_STRING)
 		LOGICAL(ans)[i] = NA_LOGICAL;
 	    else if (c1 == c2)
@@ -508,13 +599,13 @@ static SEXP string_relop(RELOP_TYPE code, SEXP s1, SEXP s2)
 		else
 		    LOGICAL(ans)[i] = (res < 0) ? 1 : 0;
 	    }
-	}
+	});
 	break;
     case GTOP:
-	for (i = 0; i < n; i++) {
+	MOD_ITERATE2(n, n1, n2, i ,i1, i2, {
 //	    if ((i+1) % NINTERRUPT == 0) R_CheckUserInterrupt();
-	    c1 = STRING_ELT(s1, i % n1);
-	    c2 = STRING_ELT(s2, i % n2);
+	    c1 = STRING_ELT(s1, i1);
+	    c2 = STRING_ELT(s2, i2);
 	    if (c1 == NA_STRING || c2 == NA_STRING)
 		LOGICAL(ans)[i] = NA_LOGICAL;
 	    else if (c1 == c2)
@@ -527,13 +618,13 @@ static SEXP string_relop(RELOP_TYPE code, SEXP s1, SEXP s2)
 		else
 		    LOGICAL(ans)[i] = (res > 0) ? 1 : 0;
 	    }
-	}
+	});
 	break;
     case LEOP:
-	for (i = 0; i < n; i++) {
+	MOD_ITERATE2(n, n1, n2, i, i1, i2, {
 //	    if ((i+1) % NINTERRUPT == 0) R_CheckUserInterrupt();
-	    c1 = STRING_ELT(s1, i % n1);
-	    c2 = STRING_ELT(s2, i % n2);
+	    c1 = STRING_ELT(s1, i1);
+	    c2 = STRING_ELT(s2, i2);
 	    if (c1 == NA_STRING || c2 == NA_STRING)
 		LOGICAL(ans)[i] = NA_LOGICAL;
 	    else if (c1 == c2)
@@ -546,13 +637,13 @@ static SEXP string_relop(RELOP_TYPE code, SEXP s1, SEXP s2)
 		else
 		    LOGICAL(ans)[i] = (res <= 0) ? 1 : 0;
 	    }
-	}
+	});
 	break;
     case GEOP:
-	for (i = 0; i < n; i++) {
+	MOD_ITERATE2(n, n1, n2, i, i1, i2, {
 //	    if ((i+1) % NINTERRUPT == 0) R_CheckUserInterrupt();
-	    c1 = STRING_ELT(s1, i % n1);
-	    c2 = STRING_ELT(s2, i % n2);
+	    c1 = STRING_ELT(s1, i1);
+	    c2 = STRING_ELT(s2, i2);
 	    if (c1 == NA_STRING || c2 == NA_STRING)
 		LOGICAL(ans)[i] = NA_LOGICAL;
 	    else if (c1 == c2)
@@ -565,7 +656,7 @@ static SEXP string_relop(RELOP_TYPE code, SEXP s1, SEXP s2)
 		else
 		    LOGICAL(ans)[i] = (res >= 0) ? 1 : 0;
 	    }
-	}
+	});
 	break;
     }
     UNPROTECT(3);
@@ -588,52 +679,52 @@ static SEXP raw_relop(RELOP_TYPE code, SEXP s1, SEXP s2)
 
     switch (code) {
     case EQOP:
-	mod_iterate(n1, n2, i1, i2) {
+	MOD_ITERATE2(n, n1, n2, i, i1, i2, {
 //	    if ((i+1) % NINTERRUPT == 0) R_CheckUserInterrupt();
 	    x1 = RAW(s1)[i1];
 	    x2 = RAW(s2)[i2];
 	    LOGICAL(ans)[i] = (x1 == x2);
-	}
+	});
 	break;
     case NEOP:
-	mod_iterate(n1, n2, i1, i2) {
+	MOD_ITERATE2(n, n1, n2, i, i1, i2, {
 //	    if ((i+1) % NINTERRUPT == 0) R_CheckUserInterrupt();
 	    x1 = RAW(s1)[i1];
 	    x2 = RAW(s2)[i2];
 	    LOGICAL(ans)[i] = (x1 != x2);
-	}
+	});
 	break;
     case LTOP:
-	mod_iterate(n1, n2, i1, i2) {
+	MOD_ITERATE2(n, n1, n2, i, i1, i2, {
 //	    if ((i+1) % NINTERRUPT == 0) R_CheckUserInterrupt();
 	    x1 = RAW(s1)[i1];
 	    x2 = RAW(s2)[i2];
 	    LOGICAL(ans)[i] = (x1 < x2);
-	}
+	});
 	break;
     case GTOP:
-	mod_iterate(n1, n2, i1, i2) {
+	MOD_ITERATE2(n, n1, n2, i, i1, i2, {
 //	    if ((i+1) % NINTERRUPT == 0) R_CheckUserInterrupt();
 	    x1 = RAW(s1)[i1];
 	    x2 = RAW(s2)[i2];
 	    LOGICAL(ans)[i] = (x1 > x2);
-	}
+	});
 	break;
     case LEOP:
-	mod_iterate(n1, n2, i1, i2) {
+	MOD_ITERATE2(n, n1, n2, i, i1, i2, {
 //	    if ((i+1) % NINTERRUPT == 0) R_CheckUserInterrupt();
 	    x1 = RAW(s1)[i1];
 	    x2 = RAW(s2)[i2];
 	    LOGICAL(ans)[i] = (x1 <= x2);
-	}
+	});
 	break;
     case GEOP:
-	mod_iterate(n1, n2, i1, i2) {
+	MOD_ITERATE2(n, n1, n2, i, i1, i2, {
 //	    if ((i+1) % NINTERRUPT == 0) R_CheckUserInterrupt();
 	    x1 = RAW(s1)[i1];
 	    x2 = RAW(s2)[i2];
 	    LOGICAL(ans)[i] = (x1 >= x2);
-	}
+	});
 	break;
     }
     UNPROTECT(2);
@@ -669,13 +760,14 @@ static SEXP bitwiseNot(SEXP a)
     if(isReal(b)) {b = PROTECT(coerceVector(b, INTSXP)); np++;} \
     if (TYPEOF(a) != TYPEOF(b)) error(_("'a' and 'b' must have the same type"));  \
     R_xlen_t i, m = XLENGTH(a), n = XLENGTH(b), mn = (m && n) ? mymax(m, n) : 0;  \
+    R_xlen_t ia, ib; \
     SEXP ans = allocVector(TYPEOF(a), mn); \
     switch(TYPEOF(a)) { \
     case INTSXP: \
-	for(i = 0; i < mn; i++) { \
-	    int aa = INTEGER(a)[i%m], bb =  INTEGER(b)[i%n]; \
+	MOD_ITERATE2(mn, m, n, i, ia, ib, { \
+	    int aa = INTEGER(a)[ia]; int bb = INTEGER(b)[ib]; \
 	    INTEGER(ans)[i] = (aa == NA_INTEGER || bb == NA_INTEGER) ? NA_INTEGER : aa op bb; \
-	} \
+	}); \
 	break; \
     default: \
 	UNIMPLEMENTED_TYPE(name, a); \
@@ -703,16 +795,17 @@ static SEXP bitwiseShiftL(SEXP a, SEXP b)
     int np = 0;
     if(isReal(a)) {a = PROTECT(coerceVector(a, INTSXP)); np++;}
     if(!isInteger(b)) {b = PROTECT(coerceVector(b, INTSXP)); np++;}
-    R_xlen_t i, m = XLENGTH(a), n = XLENGTH(b), 
+    R_xlen_t i, m = XLENGTH(a), n = XLENGTH(b),
 	mn = (m && n) ? mymax(m, n) : 0;
+    R_xlen_t ia, ib;
     SEXP ans = allocVector(TYPEOF(a), mn);
     switch(TYPEOF(a)) {
     case INTSXP:
-	for(i = 0; i < mn; i++) {
-	    int aa = INTEGER(a)[i%m], bb = INTEGER(b)[i%n];
-	    INTEGER(ans)[i] = 
+	MOD_ITERATE2(mn, m, n, i, ia, ib, {
+	    int aa = INTEGER(a)[ia]; int bb = INTEGER(b)[ib];
+	    INTEGER(ans)[i] =
 		(aa == NA_INTEGER || bb == NA_INTEGER || bb < 0 || bb > 31) ? NA_INTEGER : ((unsigned int)aa << bb);
-	}
+	});
 	break;
     default:
 	UNIMPLEMENTED_TYPE("bitShiftL", a);
@@ -726,16 +819,17 @@ static SEXP bitwiseShiftR(SEXP a, SEXP b)
     int np = 0;
     if(isReal(a)) {a = PROTECT(coerceVector(a, INTSXP)); np++;}
     if(!isInteger(b)) {b = PROTECT(coerceVector(b, INTSXP)); np++;}
-    R_xlen_t i, m = XLENGTH(a), n = XLENGTH(b), 
+    R_xlen_t i, m = XLENGTH(a), n = XLENGTH(b),
 	mn = (m && n) ? mymax(m, n) : 0;
+    R_xlen_t ia, ib;
     SEXP ans = allocVector(TYPEOF(a), mn);
     switch(TYPEOF(a)) {
     case INTSXP:
-	for(i = 0; i < mn; i++) {
-	    int aa = INTEGER(a)[i%m], bb = INTEGER(b)[i%n];
-	    INTEGER(ans)[i] = 
+	MOD_ITERATE2(mn, m, n, i, ia, ib, {
+	    int aa = INTEGER(a)[ia]; int bb = INTEGER(b)[ib];
+	    INTEGER(ans)[i] =
 		(aa == NA_INTEGER || bb == NA_INTEGER || bb < 0 || bb > 31) ? NA_INTEGER : ((unsigned int)aa >> bb);
-	}
+	});
 	break;
     default:
 	UNIMPLEMENTED_TYPE("bitShiftR", a);
