@@ -1,7 +1,7 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
  *  Copyright (C) 1995, 1996  Robert Gentleman and Ross Ihaka
- *  Copyright (C) 1997--2015  The R Core Team
+ *  Copyright (C) 1997--2016  The R Core Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -417,21 +417,33 @@ getSelectedHandler(InputHandler *handlers, fd_set *readMask)
 }
 
 
-
 #ifdef HAVE_LIBREADLINE
 
 # ifdef HAVE_READLINE_READLINE_H
 #  include <readline/readline.h>
-/* For compatibility with pre-readline4.2 systems: */
+/* For compatibility with pre-readline-4.2 systems, 
+   also missing in Apple's emulation via the NetBSD editline library.*/
 #  if !defined (_RL_FUNCTION_TYPEDEF)
 typedef void rl_vcpfunc_t (char *);
 #  endif /* _RL_FUNCTION_TYPEDEF */
+
+# if defined(RL_READLINE_VERSION) && RL_READLINE_VERSION >= 0x0603
+/* readline 6.3's rl_callback_handler_install() no longer installs
+   signal handlers, so as from that version we need an explicit
+   one. (PR#16604)  (This could have been controlled in earlier versions
+   by setting rl_catch_sigwinch.)
+ */
+#  define NEED_INT_HANDLER
+# endif
 # else
 typedef void rl_vcpfunc_t (char *);
 extern void rl_callback_handler_install(const char *, rl_vcpfunc_t *);
 extern void rl_callback_handler_remove(void);
 extern void rl_callback_read_char(void);
 extern char *tilde_expand (const char *);
+extern const char *rl_readline_name;
+/* Other externals are used, but only if the readline >= 6.3 is detected,
+   and that requires the header. */
 # endif
 
 attribute_hidden
@@ -509,6 +521,15 @@ static struct {
   rl_vcpfunc_t *fun[MAX_READLINE_NESTING];
 } ReadlineStack = {-1, MAX_READLINE_NESTING - 1};
 
+#ifdef NEED_INT_HANDLER
+static volatile Rboolean caught_sigwinch = FALSE;
+
+static RETSIGTYPE
+R_readline_sigwinch_handler(int sig)
+{
+    caught_sigwinch = TRUE;
+}
+#endif
 
 /*
   Registers the specified routine and prompt with readline
@@ -523,11 +544,42 @@ pushReadline(const char *prompt, rl_vcpfunc_t f)
      ReadlineStack.fun[++ReadlineStack.current] = f;
 
    rl_callback_handler_install(prompt, f);
+
+#ifdef NEED_INT_HANDLER
+   signal(SIGWINCH, R_readline_sigwinch_handler);
+#endif
+
    /* flush stdout in case readline wrote the prompt, but didn't flush
-      stdout to make it visible. (needed for Apple's rl in OS X 10.4-pre) */
+      stdout to make it visible. (needed for Apple's readline emulation). */
    fflush(stdout);
 }
 
+#if defined(RL_READLINE_VERSION) && RL_READLINE_VERSION >= 0x0600
+/*
+  Fix for PR#16603, for readline >= 6.0.
+
+  The readline interface is somewhat messy. readline contains the
+  function rl_free_line_state(), which its internal SIGINT handler
+  calls. However, it only cancels keyboard macros and certain other
+  things: it does not clear the line. Also, as of readline 6.3, its
+  SIGINT handler is no longer triggered during our select() loop since
+  rl_callback_handler_install() no longer installs signal handlers.
+  So we have to catch the signal and do all the work ourselves to get
+  Bash-like behavior on Ctrl-C.
+ */
+static void resetReadline(void)
+{
+    rl_free_line_state();
+    rl_cleanup_after_signal();
+    RL_UNSETSTATE(RL_STATE_ISEARCH | RL_STATE_NSEARCH | RL_STATE_VIMOTION |
+		  RL_STATE_NUMERICARG | RL_STATE_MULTIKEY);
+    /* The following two lines should be equivalent, but doing both
+       won't hurt. */
+    rl_line_buffer[rl_point = rl_end = rl_mark = 0] = 0;
+    rl_done = 1;
+}
+#endif
+
 /*
   Unregister the current readline handler and pop it from R's readline
   stack, followed by re-registering the previous one.
@@ -535,6 +587,9 @@ pushReadline(const char *prompt, rl_vcpfunc_t f)
 static void popReadline(void)
 {
   if(ReadlineStack.current > -1) {
+#if defined(RL_READLINE_VERSION) && RL_READLINE_VERSION >= 0x0600
+     resetReadline();
+#endif
      rl_callback_handler_remove();
      ReadlineStack.fun[ReadlineStack.current--] = NULL;
      if(ReadlineStack.current > -1 && ReadlineStack.fun[ReadlineStack.current])
@@ -683,9 +738,8 @@ static void initialize_rlcompletion(void)
     /* Tell the completer that we want a crack first. */
     rl_attempted_completion_function = R_custom_completion;
 
-    /* Disable sorting of possible completions; only readline >= 6 */
+    /* Disable sorting of possible completions; only readline >= 6.0 */
 #if RL_READLINE_VERSION >= 0x0600
-    /* if (rl_readline_version >= 0x0600) */
     rl_sort_completion_matches = 0;
 #endif
 
@@ -921,6 +975,14 @@ Rstd_ReadConsole(const char *prompt, unsigned char *buf, int len,
 	    if (Rg_wait_usec > 0 && (wt < 0 || wt > Rg_wait_usec))
 		wt = Rg_wait_usec;
 	    what = R_checkActivityEx(wt, 0, handleInterrupt);
+#ifdef NEED_INT_HANDLER
+            if (UsingReadline && caught_sigwinch) {
+		caught_sigwinch = FALSE;
+		// iintroduced in readline 4.0: only used for >= 6.3
+		rl_resize_terminal();
+            }
+#endif
+
 	    /* This is slightly clumsy. We have advertised the
 	     * convention that R_wait_usec == 0 means "wait forever",
 	     * but we also need to enable R_checkActivity to return
@@ -1103,7 +1165,7 @@ void attribute_hidden NORET Rstd_CleanUp(SA_TYPE saveact, int status, int runLas
 	PrintWarnings();	/* from device close and (if run) .Last */
     if(ifp) {
 	fclose(ifp);    /* input file from -f or --file= */
-	ifp = NULL;	/* To avoid trying to close it again */
+	ifp = NULL; 	/* To avoid trying to close it again */
     }
     fpu_setup(FALSE);
 
